#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.botswana import Botswana, BW, BWA
from tests.common import CommonCountryTests


class TestBotswana(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Botswana, years=range(1966, 2050), years_non_observed=range(2010, 2024))

    def test_country_aliases(self):
        self.assertAliases(Botswana, BW, BWA)

    def test_no_holidays(self):
        self.assertNoHolidays(Botswana(years=1965))

    def test_special_holidays(self):
        self.assertHoliday("2019-07-02")

    def test_new_years(self):
        for year in range(1966, 2050):
            self.assertHoliday(f"{year}-01-01", f"{year}-01-02")

        dt = (
            "2011-01-03",
            "2012-01-03",
            "2017-01-03",
            "2022-01-03",
            "2023-01-03",
        )
        self.assertHoliday(dt)
        self.assertNoNonObservedHoliday(dt)

    def test_easter(self):
        dt = (
            "2020-04-10",
            "2020-04-11",
            "2020-04-13",
            "2020-05-21",
            "2022-04-15",
            "2022-04-16",
            "2022-04-18",
            "2022-05-26",
        )
        self.assertHoliday(dt)
        self.assertNonObservedHoliday(dt)

    def test_labour_day(self):
        self.assertHoliday(f"{year}-05-01" for year in range(1966, 2050))
        dt = ("2011-05-02", "2016-05-02", "2022-05-02")
        self.assertHoliday(dt)
        self.assertNoNonObservedHoliday(dt)

        dt = ("2021-05-03", "2027-05-03", "2032-05-03")
        self.assertHolidayName("Labour Day Holiday", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_presidents_day(self):
        self.assertHoliday(
            "2019-07-15",
            "2019-07-16",
            "2020-07-20",
            "2020-07-21",
            "2021-07-19",
            "2021-07-20",
            "2022-07-18",
            "2022-07-19",
        )

    def test_botswana_day(self):
        for year in range(1966, 2050):
            self.assertHoliday(f"{year}-09-30", f"{year}-10-01")

        dt = (
            "2012-10-02",
            "2017-10-02",
            "2018-10-02",
            "2023-10-02",
        )
        self.assertHoliday(dt)
        self.assertNoNonObservedHoliday(dt)

    def test_christmas_day(self):
        for year in range(1966, 2050):
            self.assertHoliday(f"{year}-12-25", f"{year}-12-26")

        dt = (
            "2010-12-27",
            "2011-12-27",
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHoliday(dt)
        self.assertNoNonObservedHoliday(dt)

        dt = ("2020-12-28", "2026-12-28", "2037-12-28")
        self.assertHolidayName("Boxing Day Holiday", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_2021(self):
        self.assertHolidayDates(
            Botswana(years=2021),
            "2021-01-01",
            "2021-01-02",
            "2021-04-02",
            "2021-04-03",
            "2021-04-05",
            "2021-05-01",
            "2021-05-03",
            "2021-05-13",
            "2021-07-01",
            "2021-07-19",
            "2021-07-20",
            "2021-09-30",
            "2021-10-01",
            "2021-12-25",
            "2021-12-26",
            "2021-12-27",
        )

    def test_2022(self):
        self.assertHolidayDates(
            Botswana(years=2022),
            "2022-01-01",
            "2022-01-02",
            "2022-01-03",
            "2022-04-15",
            "2022-04-16",
            "2022-04-18",
            "2022-05-01",
            "2022-05-02",
            "2022-05-26",
            "2022-07-01",
            "2022-07-18",
            "2022-07-19",
            "2022-09-30",
            "2022-10-01",
            "2022-12-25",
            "2022-12-26",
            "2022-12-27",
        )

    def test_2023(self):
        self.assertHolidayDates(
            Botswana(years=2023),
            "2023-01-01",
            "2023-01-02",
            "2023-01-03",
            "2023-04-07",
            "2023-04-08",
            "2023-04-10",
            "2023-05-01",
            "2023-05-18",
            "2023-07-01",
            "2023-07-17",
            "2023-07-18",
            "2023-09-30",
            "2023-10-01",
            "2023-10-02",
            "2023-12-25",
            "2023-12-26",
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import OPTIONAL, PUBLIC
from holidays.countries.brazil import Brazil, BR, BRA
from tests.common import CommonCountryTests


class TestBrazil(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(1890, 2050)
        super().setUpClass(Brazil, years=years)
        cls.opt_holidays = Brazil(categories=OPTIONAL, years=years)

    def test_country_aliases(self):
        self.assertAliases(Brazil, BR, BRA)

    def test_no_holidays(self):
        self.assertNoHolidays(Brazil(categories=(OPTIONAL, PUBLIC), years=1889))

    def test_new_years_day(self):
        self.assertHolidayName(
            "Confraternização Universal", (f"{year}-01-01" for year in range(1890, 2050))
        )

    def test_republic_constitution_day(self):
        name = "Constituição da Republica"
        self.assertHolidayName(name, (f"{year}-02-24" for year in range(1892, 1931)))
        self.assertNoHolidayName(name, range(1890, 1892), range(1931, 2050))

    def test_discovery_of_brazil(self):
        name = "Descobrimento do Brasil"
        self.assertHolidayName(
            name, (f"{year}-05-03" for year in (*range(1890, 1931), *range(1936, 1949)))
        )
        self.assertNoHolidayName(name, range(1931, 1936), range(1949, 2050))

    def test_abolition_of_slavery_in_brazil(self):
        name = "Abolição da escravidão no Brasil"
        self.assertHolidayName(name, (f"{year}-05-13" for year in range(1890, 1931)))
        self.assertNoHolidayName(name, range(1931, 2050))

    def test_freedom_and_independence_of_american_peoples(self):
        name = "Liberdade e Independência dos Povos Americanos"
        self.assertHolidayName(name, (f"{year}-07-14" for year in range(1890, 1931)))
        self.assertNoHolidayName(name, range(1931, 2050))

    def test_good_friday(self):
        name = "Sexta-feira Santa"
        self.assertHolidayName(
            name,
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
        )
        self.assertHolidayName(name, range(1890, 2050))

    def test_tiradentes_day(self):
        name = "Tiradentes"
        self.assertHolidayName(
            name, (f"{year}-04-21" for year in set(range(1890, 2050)).difference({1931, 1932}))
        )
        self.assertNoHoliday("1931-04-21", "1932-04-21")
        self.assertNoHolidayName(name, 1931, 1932)

    def test_workers_day(self):
        name = "Dia do Trabalhador"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1925, 2050)))
        self.assertNoHoliday(f"{year}-05-01" for year in range(1890, 1925))
        self.assertNoHolidayName(name, range(1890, 1925))

    def test_independence_day(self):
        self.assertHolidayName(
            "Independência do Brasil", (f"{year}-09-07" for year in range(1890, 2050))
        )

    def test_discovery_of_america(self):
        name = "Descobrimento da América"
        self.assertHolidayName(
            name, (f"{year}-10-12" for year in (*range(1890, 1931), *range(1936, 1949)))
        )
        self.assertNoHoliday(f"{year}-10-12" for year in (*range(1931, 1936), *range(1949, 1980)))
        self.assertNoHolidayName(name, range(1931, 1936), range(1949, 2050))

    def test_our_lady_of_aparecida(self):
        name = "Nossa Senhora Aparecida"
        self.assertHolidayName(name, (f"{year}-10-12" for year in range(1980, 2050)))
        self.assertNoHolidayName(name, range(1890, 1980))

    def test_all_souls_day(self):
        self.assertHolidayName("Finados", (f"{year}-11-02" for year in range(1890, 2050)))

    def test_republic_proclamation_day(self):
        self.assertHolidayName(
            "Proclamação da República", (f"{year}-11-15" for year in range(1890, 2050))
        )

    def test_black_awareness_day(self):
        name = "Dia Nacional de Zumbi e da Consciência Negra"
        self.assertHolidayName(name, (f"{year}-11-20" for year in range(2024, 2050)))
        self.assertNoHolidayName(name, range(1890, 2024))

    def test_christmas_day(self):
        name = "Natal"
        self.assertHolidayName(name, (f"{year}-12-25" for year in range(1922, 2050)))
        self.assertNoHoliday(f"{year}-12-25" for year in range(1890, 1922))
        self.assertNoHolidayName(name, range(1890, 1922))

    def test_carnaval(self):
        name = "Carnaval"
        dt = (
            "2018-02-12",
            "2018-02-13",
            "2019-03-04",
            "2019-03-05",
            "2020-02-24",
            "2020-02-25",
            "2021-02-15",
            "2021-02-16",
            "2022-02-28",
            "2022-03-01",
            "2023-02-20",
            "2023-02-21",
            "2024-02-12",
            "2024-02-13",
        )
        self.assertHolidayName(name, self.opt_holidays, dt)
        self.assertHolidayName(name, self.opt_holidays, range(1890, 2050))
        self.assertNoHolidayName(name)
        self.assertNoHoliday(dt)

    def test_ash_wednesday(self):
        name = "Início da Quaresma"
        dt = (
            "2018-02-14",
            "2019-03-06",
            "2020-02-26",
            "2021-02-17",
            "2022-03-02",
            "2023-02-22",
            "2024-02-14",
        )
        self.assertHolidayName(name, self.opt_holidays, dt)
        self.assertHolidayName(name, self.opt_holidays, range(1890, 2050))
        self.assertNoHolidayName(name)
        self.assertNoHoliday(dt)

    def test_corpus_christi(self):
        name = "Corpus Christi"
        dt = (
            "2018-05-31",
            "2019-06-20",
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
            "2024-05-30",
        )
        self.assertHolidayName(name, self.opt_holidays, dt)
        self.assertHolidayName(name, self.opt_holidays, range(1890, 2050))
        self.assertNoHolidayName(name)
        self.assertNoHoliday(dt)

    def test_public_servants_day(self):
        name = "Dia do Servidor Público"
        self.assertHolidayName(
            name, self.opt_holidays, (f"{year}-10-28" for year in range(1890, 2050))
        )
        self.assertNoHolidayName(name)

    def test_christmas_eve(self):
        name = "Véspera de Natal"
        self.assertHolidayName(
            name, self.opt_holidays, (f"{year}-12-24" for year in range(1890, 2050))
        )
        self.assertNoHolidayName(name)

    def test_new_years_eve(self):
        name = "Véspera de Ano-Novo"
        self.assertHolidayName(
            name, self.opt_holidays, (f"{year}-12-31" for year in range(1890, 2050))
        )
        self.assertNoHolidayName(name)

    def test_ac_holidays(self):
        ac_holidays = Brazil(subdiv="AC", years=range(1995, 2050))

        name = "Dia do Evangélico"
        self.assertHolidayName(
            name,
            ac_holidays,
            (f"{year}-01-23" for year in range(2005, 2009)),
            "2013-01-25",
            "2014-01-24",
            "2018-01-26",
            "2019-01-25",
            "2020-01-24",
        )
        self.assertNoHolidayName(name, ac_holidays, 1995, range(1996, 2005))
        self.assertNoHolidayName(name)

        name = "Dia Internacional da Mulher"
        self.assertHolidayName(
            name,
            ac_holidays,
            (f"{year}-03-08" for year in range(2002, 2009)),
            "2011-03-11",
            "2012-03-09",
            "2016-03-11",
            "2017-03-10",
            "2018-03-09",
            "2022-03-11",
            "2023-03-10",
        )
        self.assertNoHolidayName(name, ac_holidays, 1995, range(1996, 2002))
        self.assertNoHolidayName(name)

        name = "Aniversário do Acre"
        self.assertHolidayName(name, ac_holidays, (f"{year}-06-15" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, ac_holidays, 1995)
        self.assertNoHolidayName(name)

        name = "Dia da Amazônia"
        self.assertHolidayName(
            name,
            ac_holidays,
            (f"{year}-09-05" for year in range(2004, 2009)),
            "2012-09-07",
            "2013-09-06",
            "2017-09-08",
            "2018-09-07",
            "2019-09-06",
        )
        self.assertNoHolidayName(name, ac_holidays, range(1996, 2004))
        self.assertNoHolidayName(name)

        name = "Assinatura do Tratado de Petrópolis"
        self.assertHolidayName(
            name,
            ac_holidays,
            (f"{year}-11-17" for year in range(1996, 2009)),
            "2009-11-20",
            "2010-11-19",
            "2011-11-18",
            "2015-11-20",
            "2016-11-18",
            "2020-11-20",
            "2021-11-19",
            "2022-11-18",
        )
        self.assertNoHolidayName(name, ac_holidays, 1995)
        self.assertNoHolidayName(name)

    def test_al_holidays(self):
        al_holidays = Brazil(subdiv="AL", years=range(1995, 2050))

        for name, dt in (
            ("São João", "06-24"),
            ("São Pedro", "06-29"),
            ("Emancipação Política de Alagoas", "09-16"),
        ):
            self.assertHolidayName(
                name, al_holidays, (f"{year}-{dt}" for year in range(1996, 2050))
            )
            self.assertNoHolidayName(name, al_holidays, 1995)
            self.assertNoHolidayName(name)

        name = "Consciência Negra"
        self.assertHolidayName(name, al_holidays, (f"{year}-11-20" for year in range(1996, 2024)))
        self.assertNoHolidayName(name, al_holidays, 1995, range(2024, 2050))
        self.assertNoHolidayName(name)

        name = "Dia do Evangélico"
        self.assertHolidayName(name, al_holidays, (f"{year}-11-30" for year in range(2013, 2050)))
        self.assertNoHolidayName(name, al_holidays, range(1995, 2013))
        self.assertNoHolidayName(name)

    def test_am_holidays(self):
        am_holidays = Brazil(subdiv="AM", years=range(1995, 2050))

        name = "Elevação do Amazonas à categoria de província"
        self.assertHolidayName(name, am_holidays, (f"{year}-09-05" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, am_holidays, 1995)
        self.assertNoHolidayName(name)

        name = "Consciência Negra"
        self.assertHolidayName(name, am_holidays, (f"{year}-11-20" for year in range(2010, 2024)))
        self.assertNoHolidayName(name, am_holidays, range(1995, 2010), range(2024, 2050))
        self.assertNoHolidayName(name)

    def test_ap_holidays(self):
        ap_holidays = Brazil(subdiv="AP", years=range(1995, 2050))

        name = "São José"
        self.assertHolidayName(name, ap_holidays, (f"{year}-03-19" for year in range(2003, 2050)))
        self.assertNoHoliday(ap_holidays, (f"{year}-03-19" for year in range(1996, 2003)))
        self.assertNoHolidayName(name, ap_holidays, range(1995, 2003))
        self.assertNoHolidayName(name)

        name = "São Tiago"
        self.assertHolidayName(name, ap_holidays, (f"{year}-07-25" for year in range(2012, 2050)))
        self.assertNoHoliday(ap_holidays, (f"{year}-07-25" for year in range(1996, 2012)))
        self.assertNoHolidayName(name, ap_holidays, range(1995, 2012))
        self.assertNoHolidayName(name)

        name = "Criação do Território Federal"
        self.assertHolidayName(name, ap_holidays, (f"{year}-09-13" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, ap_holidays, 1995)
        self.assertNoHolidayName(name)

        name = "Consciência Negra"
        self.assertHolidayName(name, ap_holidays, (f"{year}-11-20" for year in range(2008, 2024)))
        self.assertNoHoliday(ap_holidays, (f"{year}-11-20" for year in range(1996, 2008)))
        self.assertNoHolidayName(name, ap_holidays, range(1995, 2008), range(2024, 2050))
        self.assertNoHolidayName(name)

    def test_ba_holidays(self):
        ba_holidays = Brazil(subdiv="BA", years=range(1995, 2050))

        name = "Independência da Bahia"
        self.assertHolidayName(name, ba_holidays, (f"{year}-07-02" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, ba_holidays, 1995)
        self.assertNoHolidayName(name)

    def test_ce_holidays(self):
        ce_holidays = Brazil(subdiv="CE", years=range(1995, 2050))

        name = "São José"
        self.assertHolidayName(name, ce_holidays, (f"{year}-03-19" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, ce_holidays, 1995)
        self.assertNoHolidayName(name)

        name = "Abolição da escravidão no Ceará"
        self.assertHolidayName(name, ce_holidays, (f"{year}-03-25" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, ce_holidays, 1995)
        self.assertNoHolidayName(name)

        name = "Nossa Senhora da Assunção"
        self.assertHolidayName(name, ce_holidays, (f"{year}-08-15" for year in range(2004, 2050)))
        self.assertNoHoliday(ce_holidays, (f"{year}-08-15" for year in range(1996, 2004)))
        self.assertNoHolidayName(name, ce_holidays, range(1995, 2004))
        self.assertNoHolidayName(name)

    def test_df_holidays(self):
        df_holidays = Brazil(subdiv="DF", years=range(1995, 2050))

        name = "Fundação de Brasília"
        self.assertHolidayName(name, df_holidays, (f"{year}-04-21" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, df_holidays, 1995)
        self.assertNoHolidayName(name)

        name = "Dia do Evangélico"
        self.assertHolidayName(name, df_holidays, (f"{year}-11-30" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, df_holidays, 1995)
        self.assertNoHolidayName(name)

    def test_es_holidays(self):
        es_holidays = Brazil(subdiv="ES", years=range(1995, 2050))

        name = "Nossa Senhora da Penha"
        self.assertHolidayName(
            name,
            es_holidays,
            "2020-04-20",
            "2021-04-12",
            "2022-04-25",
            "2023-04-17",
            "2024-04-08",
        )
        self.assertNoHolidayName(name, es_holidays, range(1995, 2020))
        self.assertNoHolidayName(name)

    def test_go_holidays(self):
        go_holidays = Brazil(subdiv="GO", years=range(1995, 2050))

        name = "Fundação da cidade de Goiás"
        self.assertHolidayName(name, go_holidays, (f"{year}-07-26" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, go_holidays, 1995)
        self.assertNoHolidayName(name)

        name = "Pedra fundamental de Goiânia"
        self.assertHolidayName(name, go_holidays, (f"{year}-10-24" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, go_holidays, 1995)
        self.assertNoHolidayName(name)

    def test_ma_holidays(self):
        ma_holidays = Brazil(subdiv="MA", years=range(1995, 2050))

        name = "Adesão do Maranhão à independência do Brasil"
        self.assertHolidayName(name, ma_holidays, (f"{year}-07-28" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, ma_holidays, 1995)
        self.assertNoHolidayName(name)

    def test_mg_holidays(self):
        mg_holidays = Brazil(subdiv="MG", years=range(1995, 2050))

        name = "Execução de Tiradentes"
        self.assertHolidayName(name, mg_holidays, (f"{year}-04-21" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, mg_holidays, 1995)
        self.assertNoHolidayName(name)

    def test_ms_holidays(self):
        ms_holidays = Brazil(subdiv="MS", years=range(1995, 2050))

        name = "Criação do Estado"
        self.assertHolidayName(name, ms_holidays, (f"{year}-10-11" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, ms_holidays, 1995)
        self.assertNoHolidayName(name)

    def test_mt_holidays(self):
        mt_holidays = Brazil(subdiv="MT", years=range(1995, 2050))

        name = "Consciência Negra"
        self.assertHolidayName(name, mt_holidays, (f"{year}-11-20" for year in range(2003, 2024)))
        self.assertNoHoliday(mt_holidays, (f"{year}-11-20" for year in range(1996, 2003)))
        self.assertNoHolidayName(name, mt_holidays, range(1995, 2003), range(2024, 2050))
        self.assertNoHolidayName(name)

    def test_pa_holidays(self):
        pa_holidays = Brazil(subdiv="PA", years=range(1995, 2050))

        name = "Adesão do Grão-Pará à independência do Brasil"
        self.assertHolidayName(name, pa_holidays, (f"{year}-08-15" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, pa_holidays, 1995)
        self.assertNoHolidayName(name)

    def test_pb_holidays(self):
        pb_holidays = Brazil(subdiv="PB", years=range(1995, 2050))

        name = "Fundação do Estado"
        self.assertHolidayName(name, pb_holidays, (f"{year}-08-05" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, pb_holidays, 1995)
        self.assertNoHolidayName(name)

    def test_pe_holidays(self):
        pe_holidays = Brazil(subdiv="PE", years=range(1995, 2050))

        name = "Revolução Pernambucana"
        self.assertHolidayName(
            name,
            pe_holidays,
            "2008-03-02",
            "2009-03-01",
            "2018-03-04",
            "2019-03-03",
            "2020-03-01",
            "2021-03-07",
            "2022-03-06",
            "2023-03-05",
            "2024-03-03",
        )
        self.assertNoHolidayName(name, pe_holidays, range(1995, 2008))
        self.assertNoHolidayName(name)

    def test_pi_holidays(self):
        pi_holidays = Brazil(subdiv="PI", years=range(1995, 2050))

        name = "Dia do Piauí"
        self.assertHolidayName(name, pi_holidays, (f"{year}-10-19" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, pi_holidays, 1995)
        self.assertNoHolidayName(name)

    def test_pr_holidays(self):
        pr_holidays = Brazil(subdiv="PR", years=range(1995, 2050))

        name = "Emancipação do Paraná"
        self.assertHolidayName(name, pr_holidays, (f"{year}-12-19" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, pr_holidays, 1995)
        self.assertNoHolidayName(name)

    def test_rj_holidays(self):
        rj_holidays = Brazil(subdiv="RJ", years=range(1995, 2050))

        name = "São Jorge"
        self.assertHolidayName(name, rj_holidays, (f"{year}-04-23" for year in range(2008, 2050)))
        self.assertNoHoliday(rj_holidays, (f"{year}-04-23" for year in range(1996, 2008)))
        self.assertNoHolidayName(name, rj_holidays, range(1995, 2008))
        self.assertNoHolidayName(name)

        name = "Consciência Negra"
        self.assertHolidayName(name, rj_holidays, (f"{year}-11-20" for year in range(2002, 2024)))
        self.assertNoHoliday(rj_holidays, (f"{year}-11-20" for year in range(1996, 2002)))
        self.assertNoHolidayName(name, rj_holidays, range(1995, 2002), range(2024, 2050))
        self.assertNoHolidayName(name)

    def test_rn_holidays(self):
        rn_holidays = Brazil(subdiv="RN", years=range(1995, 2050))

        name = "Dia do Rio Grande do Norte"
        self.assertHolidayName(name, rn_holidays, (f"{year}-08-07" for year in range(2000, 2050)))
        self.assertNoHoliday(rn_holidays, (f"{year}-08-07" for year in range(1996, 2000)))
        self.assertNoHolidayName(name, rn_holidays, range(1995, 2000))
        self.assertNoHolidayName(name)

        name = "Mártires de Cunhaú e Uruaçu"
        self.assertHolidayName(name, rn_holidays, (f"{year}-10-03" for year in range(2007, 2050)))
        self.assertNoHoliday(rn_holidays, (f"{year}-10-03" for year in range(1996, 2007)))
        self.assertNoHolidayName(name, rn_holidays, range(1995, 2007))
        self.assertNoHolidayName(name)

    def test_ro_holidays(self):
        ro_holidays = Brazil(subdiv="RO", years=range(1995, 2050))

        name = "Criação do Estado"
        self.assertHolidayName(name, ro_holidays, (f"{year}-01-04" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, ro_holidays, 1995)
        self.assertNoHolidayName(name)

        name = "Dia do Evangélico"
        self.assertHolidayName(name, ro_holidays, (f"{year}-06-18" for year in range(2002, 2050)))
        self.assertNoHoliday(ro_holidays, (f"{year}-06-18" for year in range(1996, 2002)))
        self.assertNoHolidayName(name, ro_holidays, range(1995, 2002))
        self.assertNoHolidayName(name)

    def test_rr_holidays(self):
        rr_holidays = Brazil(subdiv="RR", years=range(1995, 2050))

        name = "Criação do Estado"
        self.assertHolidayName(name, rr_holidays, (f"{year}-10-05" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, rr_holidays, 1995)
        self.assertNoHolidayName(name)

    def test_rs_holidays(self):
        rs_holidays = Brazil(subdiv="RS", years=range(1995, 2050))

        name = "Dia do Gaúcho"
        self.assertHolidayName(name, rs_holidays, (f"{year}-09-20" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, rs_holidays, 1995)
        self.assertNoHolidayName(name)

    def test_sc_holidays(self):
        sc_holidays = Brazil(subdiv="SC", years=range(1995, 2050))

        name = "Dia do Estado de Santa Catarina"
        self.assertHolidayName(
            name,
            sc_holidays,
            "2004-08-11",
            "2005-08-14",
            "2006-08-13",
            "2007-08-12",
            "2018-08-12",
            "2019-08-11",
            "2020-08-16",
            "2021-08-15",
            "2022-08-14",
            "2023-08-13",
            "2024-08-11",
        )
        self.assertHolidayName(name, sc_holidays, range(2004, 2050))
        self.assertNoHolidayName(name, sc_holidays, range(1995, 2004))
        self.assertNoHolidayName(name)

        name = "Dia de Santa Catarina de Alexandria"
        self.assertHoliday(
            sc_holidays,
            "1996-11-25",
            "1997-11-25",
            "1998-11-25",
            "1999-11-28",
            "2000-11-26",
            "2004-11-25",
            "2005-11-27",
            "2018-11-25",
            "2019-12-01",
            "2020-11-29",
            "2021-11-28",
            "2022-11-27",
            "2023-11-26",
            "2024-12-01",
        )
        self.assertHolidayName(name, sc_holidays, range(1996, 2050))
        self.assertNoHolidayName(name, sc_holidays, 1995)
        self.assertNoHolidayName(name)

    def test_se_holidays(self):
        se_holidays = Brazil(subdiv="SE", years=range(1995, 2050))

        name = "Emancipação política de Sergipe"
        self.assertHolidayName(name, se_holidays, (f"{year}-07-08" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, se_holidays, 1995)
        self.assertNoHolidayName(name)

    def test_sp_holidays(self):
        sp_holidays = Brazil(subdiv="SP", years=range(1995, 2050))

        name = "Revolução Constitucionalista"
        self.assertHolidayName(name, sp_holidays, (f"{year}-07-09" for year in range(1997, 2050)))
        self.assertNoHolidayName(name, sp_holidays, 1995, 1996)
        self.assertNoHolidayName(name)

    def test_to_holidays(self):
        to_holidays = Brazil(subdiv="TO", years=range(1995, 2050))

        name = "Dia da Autonomia"
        self.assertHolidayName(name, to_holidays, (f"{year}-03-18" for year in range(1998, 2050)))
        self.assertNoHoliday(to_holidays, (f"{year}-03-18" for year in range(1996, 1998)))
        self.assertNoHolidayName(name, to_holidays, range(1995, 1998))
        self.assertNoHolidayName(name)

        name = "Nossa Senhora da Natividade"
        self.assertHolidayName(name, to_holidays, (f"{year}-09-08" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, to_holidays, 1995)
        self.assertNoHolidayName(name)

        name = "Criação do Estado"
        self.assertHolidayName(name, to_holidays, (f"{year}-10-05" for year in range(1996, 2050)))
        self.assertNoHolidayName(name, to_holidays, 1995)
        self.assertNoHolidayName(name)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Confraternização Universal"),
            ("2023-01-04", "Criação do Estado"),
            ("2023-01-23", "Dia do Evangélico"),
            ("2023-02-20", "Carnaval"),
            ("2023-02-21", "Carnaval"),
            ("2023-02-22", "Início da Quaresma"),
            ("2023-03-05", "Revolução Pernambucana"),
            ("2023-03-10", "Dia Internacional da Mulher"),
            ("2023-03-18", "Dia da Autonomia"),
            ("2023-03-19", "São José"),
            ("2023-03-25", "Abolição da escravidão no Ceará"),
            ("2023-04-07", "Sexta-feira Santa"),
            ("2023-04-17", "Nossa Senhora da Penha"),
            ("2023-04-21", "Execução de Tiradentes; Fundação de Brasília; Tiradentes"),
            ("2023-04-23", "São Jorge"),
            ("2023-05-01", "Dia do Trabalhador"),
            ("2023-06-08", "Corpus Christi"),
            ("2023-06-15", "Aniversário do Acre"),
            ("2023-06-18", "Dia do Evangélico"),
            ("2023-06-24", "São João"),
            ("2023-06-29", "São Pedro"),
            ("2023-07-02", "Independência da Bahia"),
            ("2023-07-08", "Emancipação política de Sergipe"),
            ("2023-07-09", "Revolução Constitucionalista"),
            ("2023-07-25", "São Tiago"),
            ("2023-07-26", "Fundação da cidade de Goiás"),
            ("2023-07-28", "Adesão do Maranhão à independência do Brasil"),
            ("2023-08-05", "Fundação do Estado"),
            ("2023-08-07", "Dia do Rio Grande do Norte"),
            ("2023-08-13", "Dia do Estado de Santa Catarina"),
            (
                "2023-08-15",
                "Adesão do Grão-Pará à independência do Brasil; Nossa Senhora da Assunção",
            ),
            ("2023-09-05", "Elevação do Amazonas à categoria de província"),
            ("2023-09-07", "Independência do Brasil"),
            ("2023-09-08", "Dia da Amazônia; Nossa Senhora da Natividade"),
            ("2023-09-13", "Criação do Território Federal"),
            ("2023-09-16", "Emancipação Política de Alagoas"),
            ("2023-09-20", "Dia do Gaúcho"),
            ("2023-10-03", "Mártires de Cunhaú e Uruaçu"),
            ("2023-10-05", "Criação do Estado"),
            ("2023-10-11", "Criação do Estado"),
            ("2023-10-12", "Nossa Senhora Aparecida"),
            ("2023-10-19", "Dia do Piauí"),
            ("2023-10-24", "Pedra fundamental de Goiânia"),
            ("2023-10-28", "Dia do Servidor Público"),
            ("2023-11-02", "Finados"),
            ("2023-11-15", "Proclamação da República"),
            ("2023-11-17", "Assinatura do Tratado de Petrópolis"),
            ("2023-11-20", "Consciência Negra"),
            ("2023-11-26", "Dia de Santa Catarina de Alexandria"),
            ("2023-11-30", "Dia do Evangélico"),
            ("2023-12-19", "Emancipação do Paraná"),
            ("2023-12-24", "Véspera de Natal"),
            ("2023-12-25", "Natal"),
            ("2023-12-31", "Véspera de Ano-Novo"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "Universal Fraternization Day"),
            ("2023-01-04", "State Creation Day"),
            ("2023-01-23", "Evangelical Day"),
            ("2023-02-20", "Carnival"),
            ("2023-02-21", "Carnival"),
            ("2023-02-22", "Ash Wednesday"),
            ("2023-03-05", "Pernambuco Revolution"),
            ("2023-03-10", "International Women's Day"),
            ("2023-03-18", "Autonomy Day"),
            ("2023-03-19", "Saint Joseph's Day"),
            ("2023-03-25", "Abolition of slavery in Ceará"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-17", "Our Lady of Penha"),
            ("2023-04-21", "Founding of Brasilia; Tiradentes' Day; Tiradentes' Execution"),
            ("2023-04-23", "Saint George's Day"),
            ("2023-05-01", "Worker's Day"),
            ("2023-06-08", "Corpus Christi"),
            ("2023-06-15", "Founding of Acre"),
            ("2023-06-18", "Evangelical Day"),
            ("2023-06-24", "Saint John's Day"),
            ("2023-06-29", "Saint Peter's Day"),
            ("2023-07-02", "Bahia Independence Day"),
            ("2023-07-08", "Sergipe Political Emancipation Day"),
            ("2023-07-09", "Constitutionalist Revolution"),
            ("2023-07-25", "Saint James' Day"),
            ("2023-07-26", "Foundation of Goiás city"),
            ("2023-07-28", "Maranhão joining to independence of Brazil"),
            ("2023-08-05", "State Founding Day"),
            ("2023-08-07", "Rio Grande do Norte Day"),
            ("2023-08-13", "Santa Catarina State Day"),
            ("2023-08-15", "Grão-Pará joining to independence of Brazil; Our Lady of Assumption"),
            ("2023-09-05", "Elevation of Amazonas to province"),
            ("2023-09-07", "Independence Day"),
            ("2023-09-08", "Amazonia Day; Our Lady of Nativity"),
            ("2023-09-13", "Creation of the Federal Territory"),
            ("2023-09-16", "Political Emancipation of Alagoas"),
            ("2023-09-20", "Gaucho Day"),
            ("2023-10-03", "Uruaçu and Cunhaú Martyrs Day"),
            ("2023-10-05", "State Creation Day"),
            ("2023-10-11", "State Creation Day"),
            ("2023-10-12", "Our Lady of Aparecida"),
            ("2023-10-19", "Piauí Day"),
            ("2023-10-24", "Foundation of Goiânia"),
            ("2023-10-28", "Public Servant's Day"),
            ("2023-11-02", "All Souls' Day"),
            ("2023-11-15", "Republic Proclamation Day"),
            ("2023-11-17", "Signing of the Petropolis Treaty"),
            ("2023-11-20", "Black Awareness Day"),
            ("2023-11-26", "Saint Catherine of Alexandria Day"),
            ("2023-11-30", "Evangelical Day"),
            ("2023-12-19", "Political Emancipation of Paraná"),
            ("2023-12-24", "Christmas Eve"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-31", "New Year's Eve"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2023-01-01", "День всесвітнього братання"),
            ("2023-01-04", "День створення штату"),
            ("2023-01-23", "Євангельський день"),
            ("2023-02-20", "Карнавал"),
            ("2023-02-21", "Карнавал"),
            ("2023-02-22", "Попільна середа"),
            ("2023-03-05", "День Пернамбуканської революції"),
            ("2023-03-10", "Міжнародний жіночий день"),
            ("2023-03-18", "День автономії"),
            ("2023-03-19", "День Святого Йосипа"),
            ("2023-03-25", "День скасування рабства в Сеарі"),
            ("2023-04-07", "Страсна пʼятниця"),
            ("2023-04-17", "День Богоматері Пенья"),
            ("2023-04-21", "День Тирадентіса; День заснування Бразиліа; День страти Тирадентіса"),
            ("2023-04-23", "День Святого Георгія"),
            ("2023-05-01", "День трудящих"),
            ("2023-06-08", "Свято Тіла і Крові Христових"),
            ("2023-06-15", "День заснування Акрі"),
            ("2023-06-18", "Євангельський день"),
            ("2023-06-24", "День Святого Івана"),
            ("2023-06-29", "День Святого Петра"),
            ("2023-07-02", "День незалежності Баїї"),
            ("2023-07-08", "День політичного звільнення Сержипі"),
            ("2023-07-09", "День Конституціоналістської революції"),
            ("2023-07-25", "День Святого Якова"),
            ("2023-07-26", "День заснування міста Гояс"),
            ("2023-07-28", "День приєдання Мараньяна до незалежності Бразилії"),
            ("2023-08-05", "День заснування штату"),
            ("2023-08-07", "День Ріо-Гранді-ду-Норті"),
            ("2023-08-13", "День штату Санта-Катарина"),
            (
                "2023-08-15",
                "День Богоматері Внебовзяття; День приєдання Гран-Пара до незалежності Бразилії",
            ),
            ("2023-09-05", "День піднесення Амазонас до категорії провінцій"),
            ("2023-09-07", "День незалежності Бразилії"),
            ("2023-09-08", "День Амазонії; День Богоматері Різдва"),
            ("2023-09-13", "День створення федеральної території"),
            ("2023-09-16", "День політичного звільнення Алагоаса"),
            ("2023-09-20", "День Гаучо"),
            ("2023-10-03", "День мучеників Куньяу та Уруасу"),
            ("2023-10-05", "День створення штату"),
            ("2023-10-11", "День створення штату"),
            ("2023-10-12", "День Богоматері Апаресіди"),
            ("2023-10-19", "День Піауї"),
            ("2023-10-24", "День заснування Гоянії"),
            ("2023-10-28", "День громадського службовця"),
            ("2023-11-02", "День усіх померлих"),
            ("2023-11-15", "День проголошення республіки"),
            ("2023-11-17", "День підписання Петрополіського договору"),
            ("2023-11-20", "День свідомості темношкірих"),
            ("2023-11-26", "День Святої Катерини Александрійської"),
            ("2023-11-30", "Євангельський день"),
            ("2023-12-19", "День політичного звільнення Парани"),
            ("2023-12-24", "Святий вечір"),
            ("2023-12-25", "Різдво Христове"),
            ("2023-12-31", "Переддень Нового року"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from typing import Optional

from holidays.calendars.gregorian import _timedelta


class EasternCalendarHolidays:
    """
    Eastern calendar holidays base class.
    """

    def _add_eastern_calendar_holiday(
        self,
        name: str,
        dt_estimated: tuple[Optional[date], bool],
        show_estimated: bool = True,
        days_delta: int = 0,
    ) -> Optional[date]:
        """
        Add Eastern (Buddhist, Chinese, Hindu, Islamic) calendar holiday.

        Adds customizable estimation label to holiday name if holiday date is an estimation.
        """
        estimated_label = getattr(self, "estimated_label", "%s (estimated)")
        dt, is_estimated = dt_estimated

        if days_delta and dt:
            dt = _timedelta(dt, days_delta)

        return (
            self._add_holiday(
                self.tr(estimated_label) % self.tr(name)
                if is_estimated and show_estimated
                else name,
                dt,
            )
            if dt
            else None
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import GOVERNMENT, UNOFFICIAL
from holidays.countries.united_states import UnitedStates, US, USA
from tests.common import CommonCountryTests


class TestUnitedStates(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(1865, 2050)
        super().setUpClass(UnitedStates, years=years, years_non_observed=range(2000, 2024))
        cls.state_hols = {
            subdiv: UnitedStates(subdiv=subdiv, years=years)
            for subdiv in UnitedStates.subdivisions
        }
        cls.gov_hols = UnitedStates(categories=GOVERNMENT, years=years)
        cls.gov_hols_non_obs = UnitedStates(observed=False, categories=GOVERNMENT, years=years)
        cls.unofficial_hols = UnitedStates(categories=UNOFFICIAL, years=years)

    def test_country_aliases(self):
        self.assertAliases(UnitedStates, US, USA)

    def test_subdiv_deprecation(self):
        self.assertDeprecatedSubdivisions("This subdivision is deprecated and will be removed")

    def test_new_years_day(self):
        name = "New Year's Day"
        obs_dt = (
            "2004-12-31",
            "2006-01-02",
            "2010-12-31",
            "2012-01-02",
            "2017-01-02",
            "2021-12-31",
            "2023-01-02",
        )
        for holidays, holidays_non_obs in (
            (self.holidays, self.holidays_non_observed),
            (self.gov_hols, self.gov_hols_non_obs),
        ):
            self.assertHolidayName(name, holidays, (f"{year}-01-01" for year in range(1871, 2050)))
            self.assertNoHoliday(holidays, (f"{year}-01-01" for year in range(1865, 1871)))
            self.assertNoHolidayName(name, holidays, range(1865, 1871))
            self.assertHolidayName(f"{name} (observed)", holidays, obs_dt)
            self.assertNoNonObservedHoliday(holidays_non_obs, obs_dt)

    def test_memorial_day(self):
        name = "Memorial Day"
        self.assertHolidayName(name, (f"{year}-05-30" for year in range(1888, 1971)))
        self.assertNoHolidayName(name, range(1865, 1888))
        self.assertNoHolidayName(name, self.gov_hols, range(1865, 1971))
        dts = (
            "1971-05-31",
            "1972-05-29",
            "2010-05-31",
            "2011-05-30",
            "2012-05-28",
            "2013-05-27",
            "2014-05-26",
            "2015-05-25",
            "2016-05-30",
            "2017-05-29",
            "2018-05-28",
            "2019-05-27",
            "2020-05-25",
            "2021-05-31",
            "2022-05-30",
            "2023-05-29",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, self.gov_hols, dts)
        self.assertNoHoliday("1971-05-30", "1972-05-30")
        self.assertNoHoliday(self.gov_hols, "1971-05-30", "1972-05-30")

    def test_juneteenth_day(self):
        name = "Juneteenth National Independence Day"
        obs_dt = (
            "2021-06-18",
            "2022-06-20",
            "2027-06-18",
            "2032-06-18",
        )
        for holidays, holidays_non_obs in (
            (self.holidays, self.holidays_non_observed),
            (self.gov_hols, self.gov_hols_non_obs),
        ):
            self.assertHolidayName(name, holidays, (f"{year}-06-19" for year in range(2021, 2050)))
            self.assertNoHoliday(holidays, (f"{year}-06-19" for year in range(1865, 2021)))
            self.assertNoHolidayName(name, holidays, range(1865, 2021))
            self.assertHolidayName(f"{name} (observed)", holidays, obs_dt)
            self.assertNoNonObservedHoliday(holidays_non_obs, obs_dt)

    def test_independence_day(self):
        name = "Independence Day"
        obs_dt = (
            "2004-07-05",
            "2009-07-03",
            "2010-07-05",
            "2015-07-03",
            "2020-07-03",
            "2021-07-05",
        )
        for holidays, holidays_non_obs in (
            (self.holidays, self.holidays_non_observed),
            (self.gov_hols, self.gov_hols_non_obs),
        ):
            self.assertHolidayName(name, holidays, (f"{year}-07-04" for year in range(1871, 2050)))
            self.assertNoHoliday(holidays, (f"{year}-07-04" for year in range(1865, 1871)))
            self.assertNoHolidayName(name, holidays, range(1865, 1871))
            self.assertHolidayName(f"{name} (observed)", holidays, obs_dt)
            self.assertNoNonObservedHoliday(holidays_non_obs, obs_dt)

    def test_labor_day(self):
        name = "Labor Day"
        dts = (
            "2010-09-06",
            "2011-09-05",
            "2012-09-03",
            "2013-09-02",
            "2014-09-01",
            "2015-09-07",
            "2016-09-05",
            "2017-09-04",
            "2018-09-03",
            "2019-09-02",
            "2020-09-07",
            "2021-09-06",
            "2022-09-05",
            "2023-09-04",
        )
        for holidays in (self.holidays, self.gov_hols):
            self.assertHolidayName(name, holidays, range(1894, 2050))
            self.assertNoHolidayName(name, holidays, range(1865, 1894))
            self.assertHolidayName(name, holidays, dts)

    def test_veterans_day(self):
        name_1 = "Armistice Day"
        name_2 = "Veterans Day"
        dts = (
            "1971-10-25",
            "1972-10-23",
            "1973-10-22",
            "1974-10-28",
            "1975-10-27",
            "1976-10-25",
            "1977-10-24",
        )
        obs_dt = (
            "2000-11-10",
            "2001-11-12",
            "2006-11-10",
            "2007-11-12",
            "2012-11-12",
            "2017-11-10",
            "2018-11-12",
            "2023-11-10",
        )
        for holidays, holidays_non_obs in (
            (self.holidays, self.holidays_non_observed),
            (self.gov_hols, self.gov_hols_non_obs),
        ):
            self.assertHolidayName(
                name_1, holidays, (f"{year}-11-11" for year in range(1938, 1954))
            )
            self.assertHolidayName(
                name_2,
                holidays,
                (f"{year}-11-11" for year in (*range(1954, 1971), *range(1978, 2050))),
            )
            self.assertNoHolidayName(name_1, holidays, range(1865, 1938), range(1954, 2050))
            self.assertNoHolidayName(name_2, holidays, range(1865, 1954))
            self.assertHolidayName(name_2, holidays, dts)
            self.assertHolidayName(f"{name_2} (observed)", holidays, obs_dt)
            self.assertNoNonObservedHoliday(holidays_non_obs, obs_dt)

    def test_thanksgiving_day(self):
        name = "Thanksgiving Day"
        dts = (
            "2010-11-25",
            "2011-11-24",
            "2012-11-22",
            "2013-11-28",
            "2014-11-27",
            "2015-11-26",
            "2016-11-24",
            "2017-11-23",
            "2018-11-22",
            "2019-11-28",
            "2020-11-26",
            "2021-11-25",
            "2022-11-24",
            "2023-11-23",
        )
        for holidays in (self.holidays, self.gov_hols):
            self.assertHolidayName(name, holidays, range(1871, 2050))
            self.assertNoHolidayName(name, holidays, range(1865, 1871))
            self.assertHolidayName(name, holidays, dts)

    def test_christmas_day(self):
        name = "Christmas Day"
        obs_dt = (
            "2004-12-24",
            "2005-12-26",
            "2010-12-24",
            "2011-12-26",
            "2016-12-26",
            "2021-12-24",
            "2022-12-26",
        )
        for holidays, holidays_non_obs in (
            (self.holidays, self.holidays_non_observed),
            (self.gov_hols, self.gov_hols_non_obs),
        ):
            self.assertHolidayName(name, holidays, (f"{year}-12-25" for year in range(1871, 2050)))
            self.assertNoHoliday(holidays, (f"{year}-12-25" for year in range(1865, 1871)))
            self.assertNoHolidayName(name, holidays, range(1865, 1871))
            self.assertHolidayName(f"{name} (observed)", holidays, obs_dt)
            self.assertNoNonObservedHoliday(holidays_non_obs, obs_dt)

    def test_martin_luther_king_day(self):
        name = "Martin Luther King Jr. Day"
        name_gov = "Birthday of Martin Luther King, Jr."
        dts = (
            "1986-01-20",
            "2010-01-18",
            "2011-01-17",
            "2012-01-16",
            "2013-01-21",
            "2014-01-20",
            "2015-01-19",
            "2016-01-18",
            "2017-01-16",
            "2018-01-15",
            "2019-01-21",
            "2020-01-20",
            "2021-01-18",
            "2022-01-17",
            "2023-01-16",
        )
        for holidays, holiday_name in (
            (self.holidays, name),
            (self.gov_hols, name_gov),
        ):
            self.assertHolidayName(holiday_name, holidays, range(1986, 2050))
            self.assertNoHolidayName(holiday_name, holidays, range(1865, 1986))
            self.assertHolidayName(holiday_name, holidays, dts)

        for subdiv, holidays in self.state_hols.items():
            if subdiv not in {"AL", "AR", "AZ", "GA", "ID", "MS", "NH"}:
                self.assertHolidayName(name, holidays, dts)
                self.assertHolidayName(name, holidays, range(1986, 2050))
                self.assertNoHolidayName(name, holidays, range(1865, 1986))

    def test_martin_luther_king_day_states(self):
        dts = (
            "1986-01-20",
            "2010-01-18",
            "2011-01-17",
            "2012-01-16",
            "2013-01-21",
            "2014-01-20",
            "2015-01-19",
            "2016-01-18",
            "2017-01-16",
            "2018-01-15",
            "2019-01-21",
            "2020-01-20",
            "2021-01-18",
            "2022-01-17",
            "2023-01-16",
        )

        al_name = "Martin Luther King, Jr & Robert E. Lee's Birthday"
        al_holidays = self.state_hols["AL"]
        self.assertHolidayName(al_name, al_holidays, dts)
        self.assertHolidayName(al_name, al_holidays, range(1986, 2050))
        self.assertNoHolidayName(al_name, al_holidays, range(1865, 1986))

        ar_name_1 = "Dr. Martin Luther King Jr. and Robert E. Lee's Birthdays"
        ar_name_2 = "Martin Luther King Jr. Day"
        ar_holidays = self.state_hols["AR"]
        self.assertHolidayName(
            ar_name_1,
            ar_holidays,
            "2010-01-18",
            "2011-01-17",
            "2012-01-16",
            "2013-01-21",
            "2014-01-20",
            "2015-01-19",
            "2016-01-18",
            "2017-01-16",
        )
        self.assertHolidayName(
            ar_name_2,
            ar_holidays,
            "2018-01-15",
            "2019-01-21",
            "2020-01-20",
            "2021-01-18",
            "2022-01-17",
            "2023-01-16",
        )
        self.assertHolidayName(ar_name_1, ar_holidays, range(1986, 2018))
        self.assertHolidayName(ar_name_2, ar_holidays, range(2018, 2050))
        self.assertNoHolidayName(ar_name_1, ar_holidays, range(1865, 1986), range(2018, 2050))
        self.assertNoHolidayName(ar_name_2, ar_holidays, range(1865, 2018))

        az_name = "Dr. Martin Luther King Jr. / Civil Rights Day"
        az_holidays = self.state_hols["AZ"]
        self.assertHolidayName(az_name, az_holidays, dts)
        self.assertHolidayName(az_name, az_holidays, range(1986, 2050))
        self.assertNoHolidayName(az_name, az_holidays, range(1865, 1986))

        ga_name_1 = "Robert E. Lee's Birthday"
        ga_name_2 = "Martin Luther King Jr. Day"
        ga_holidays = self.state_hols["GA"]
        self.assertHolidayName(
            ga_name_1,
            ga_holidays,
            "2010-01-18",
            "2011-01-17",
        )
        self.assertHolidayName(
            ga_name_2,
            ga_holidays,
            "2012-01-16",
            "2013-01-21",
            "2014-01-20",
            "2015-01-19",
            "2016-01-18",
            "2017-01-16",
            "2018-01-15",
            "2019-01-21",
            "2020-01-20",
            "2021-01-18",
            "2022-01-17",
            "2023-01-16",
        )
        self.assertHolidayName(ga_name_1, ga_holidays, range(1986, 2012))
        self.assertHolidayName(ga_name_2, ga_holidays, range(2012, 2050))
        self.assertNoHolidayName(ga_name_2, ga_holidays, range(1865, 2012))

        id_name_1 = "Martin Luther King Jr. Day"
        id_name_2 = "Martin Luther King Jr. / Idaho Human Rights Day"
        id_holidays = self.state_hols["ID"]
        self.assertHolidayName(
            id_name_1,
            id_holidays,
            "2000-01-17",
            "2001-01-15",
            "2002-01-21",
            "2003-01-20",
            "2004-01-19",
            "2005-01-17",
        )
        self.assertHolidayName(
            id_name_2,
            id_holidays,
            "2006-01-16",
            "2010-01-18",
            "2011-01-17",
            "2012-01-16",
            "2013-01-21",
            "2014-01-20",
            "2015-01-19",
            "2016-01-18",
            "2017-01-16",
            "2018-01-15",
            "2019-01-21",
            "2020-01-20",
            "2021-01-18",
            "2022-01-17",
            "2023-01-16",
        )
        self.assertHolidayName(id_name_1, id_holidays, range(1986, 2006))
        self.assertHolidayName(id_name_2, id_holidays, range(2006, 2050))
        self.assertNoHolidayName(id_name_1, id_holidays, range(1865, 1986), range(2006, 2050))
        self.assertNoHolidayName(id_name_2, id_holidays, range(1865, 2006))

        ms_name = "Dr. Martin Luther King Jr. and Robert E. Lee's Birthdays"
        ms_holidays = self.state_hols["MS"]
        self.assertHolidayName(ms_name, ms_holidays, dts)
        self.assertHolidayName(ms_name, ms_holidays, range(1986, 2050))
        self.assertNoHolidayName(ms_name, ms_holidays, range(1865, 1986))

        nh_name = "Dr. Martin Luther King Jr. / Civil Rights Day"
        nh_holidays = self.state_hols["NH"]
        self.assertHolidayName(nh_name, nh_holidays, dts)
        self.assertHolidayName(nh_name, nh_holidays, range(1986, 2050))
        self.assertNoHolidayName(nh_name, nh_holidays, range(1865, 1986))

    def test_washingtons_birthday(self):
        name = "Washington's Birthday"
        dts = (
            "2010-02-15",
            "2011-02-21",
            "2012-02-20",
            "2013-02-18",
            "2014-02-17",
            "2015-02-16",
            "2016-02-15",
            "2017-02-20",
            "2018-02-19",
            "2019-02-18",
            "2020-02-17",
            "2021-02-15",
            "2022-02-21",
            "2023-02-20",
        )
        for holidays in (self.holidays, self.gov_hols):
            self.assertHolidayName(name, holidays, (f"{year}-02-22" for year in range(1879, 1971)))
            self.assertHolidayName(name, holidays, dts)
            self.assertHolidayName(name, holidays, range(1971, 2050))
            self.assertNoHolidayName(name, holidays, range(1865, 1879))

        for subdiv, holidays in self.state_hols.items():
            if subdiv not in {
                "AK",
                "AL",
                "AR",
                "AZ",
                "CA",
                "CO",
                "DE",
                "FL",
                "GA",
                "HI",
                "ID",
                "IN",
                "MD",
                "MN",
                "MT",
                "NJ",
                "NM",
                "OH",
                "OK",
                "OR",
                "PA",
                "PR",
                "SC",
                "TN",
                "TX",
                "UT",
                "VA",
                "VI",
                "VT",
                "WA",
                "WV",
                "WY",
            }:
                self.assertHolidayName(
                    name, holidays, (f"{year}-02-22" for year in range(1879, 1971))
                )
                self.assertHolidayName(name, holidays, dts)
                self.assertHolidayName(name, holidays, range(1971, 2050))
                self.assertNoHolidayName(name, holidays, range(1865, 1879))

    def test_washingtons_birthday_states(self):
        dts = (
            "2010-02-15",
            "2011-02-21",
            "2012-02-20",
            "2013-02-18",
            "2014-02-17",
            "2015-02-16",
            "2016-02-15",
            "2017-02-20",
            "2018-02-19",
            "2019-02-18",
            "2020-02-17",
            "2021-02-15",
            "2022-02-21",
            "2023-02-20",
        )

        for subdiv, name in (
            ("AK", "Presidents' Day"),
            ("AL", "George Washington & Thomas Jefferson's Birthday"),
            ("AR", "George Washington's Birthday and Daisy Gatson Bates Day"),
            ("AZ", "Lincoln/Washington Presidents' Day"),
            ("CA", "Presidents' Day"),
            ("CO", "Washington-Lincoln Day"),
            ("DE", None),
            ("HI", "Presidents' Day"),
            ("ID", "Presidents' Day"),
            ("MD", "Presidents' Day"),
            ("MN", "Washington's and Lincoln's Birthday"),
            ("MT", "Lincoln's and Washington's Birthdays"),
            ("NJ", "Presidents Day"),
            ("NM", None),
            ("OH", "Washington-Lincoln Day"),
            ("OK", "Presidents' Day"),
            ("OR", "Presidents Day"),
            ("PA", "Presidents' Day"),
            ("PR", "Presidents' Day"),
            ("SC", "President's Day"),
            ("TN", "President's Day"),
            ("TX", "Presidents' Day"),
            ("UT", "Washington and Lincoln Day"),
            ("VA", "George Washington Day"),
            ("VI", "Presidents' Day"),
            ("VT", "Presidents' Day"),
            ("WA", "Presidents' Day"),
            ("WV", "Presidents' Day"),
            ("WY", "President's Day"),
        ):
            if name:
                self.assertHolidayName(name, self.state_hols[subdiv], dts)
                self.assertHolidayName(name, self.state_hols[subdiv], range(1971, 2050))
                self.assertNoHolidayName(name, self.state_hols[subdiv], range(1865, 1879))
            else:
                self.assertNoHoliday(self.state_hols[subdiv], dts)
                self.assertNoHolidayName(name, self.state_hols[subdiv])

        name = "Washington's Birthday"
        fl_holidays = self.state_hols["FL"]
        self.assertNoHoliday(
            fl_holidays,
            "2010-02-15",
            "2011-02-21",
            "2012-02-20",
            "2013-02-18",
            "2014-02-17",
            "2015-02-16",
            "2017-02-20",
            "2018-02-19",
            "2019-02-18",
            "2020-02-17",
            "2022-02-21",
            "2023-02-20",
        )
        self.assertNoHolidayName(name, fl_holidays)

        dts = (
            "2010-12-23",
            "2011-12-23",
            "2012-12-24",
            "2013-12-24",
            "2014-12-26",
            "2015-12-24",
            "2016-12-23",
            "2017-12-26",
            "2018-12-24",
            "2019-12-24",
            "2020-12-24",
            "2021-12-23",
            "2022-12-23",
            "2023-12-26",
            "2024-12-24",
        )
        for subdiv in ("GA", "IN"):
            self.assertHolidayName(name, self.state_hols[subdiv], dts)
            self.assertHolidayName(name, self.state_hols[subdiv], range(1971, 2050))
            self.assertNoHolidayName(name, self.state_hols[subdiv], range(1865, 1879))

    def test_columbus_day(self):
        name = "Columbus Day"
        dts = (
            "2010-10-11",
            "2011-10-10",
            "2012-10-08",
            "2013-10-14",
            "2014-10-13",
            "2015-10-12",
            "2016-10-10",
            "2017-10-09",
            "2018-10-08",
            "2019-10-14",
            "2020-10-12",
            "2021-10-11",
            "2022-10-10",
            "2023-10-09",
        )
        self.assertHolidayName(name, (f"{year}-10-12" for year in range(1937, 1971)))
        self.assertHolidayName(name, range(1971, 2050))
        self.assertNoHolidayName(name, range(1865, 1937))
        self.assertHolidayName(name, dts)

        self.assertHolidayName(name, self.gov_hols, range(1971, 2050))
        self.assertNoHolidayName(name, self.gov_hols, range(1865, 1937))
        self.assertHolidayName(name, self.gov_hols, dts)

        subdivs_have_columbus_day = {
            "AS",
            "AZ",
            "CT",
            "GA",
            "ID",
            "IL",
            "IN",
            "MA",
            "MD",
            "MO",
            "MT",
            "NJ",
            "NY",
            "OH",
            "PA",
            "UT",
            "WV",
        }
        subdivs_have_columbus_day_with_other_name = {
            "AK",
            "AL",
            "CA",
            "DC",
            "ME",
            "MP",
            "NE",
            "NM",
            "RI",
            "SD",
            "VA",
            "VI",
        }

        for subdiv in subdivs_have_columbus_day:
            self.assertHolidayName(name, self.state_hols[subdiv], dts)
            self.assertHolidayName(name, self.state_hols[subdiv], range(1971, 2050))
            self.assertNoHolidayName(name, self.state_hols[subdiv], range(1865, 1937))

        for subdiv in (
            set(UnitedStates.subdivisions)
            - subdivs_have_columbus_day
            - subdivs_have_columbus_day_with_other_name
        ):
            self.assertNoHolidayName(name, self.state_hols[subdiv])
            self.assertNoHoliday(self.state_hols[subdiv], dts)

    def test_columbus_day_states(self):
        name_1 = "Columbus Day"
        name_2 = "Indigenous Peoples' Day"
        dts = (
            "2010-10-11",
            "2011-10-10",
            "2012-10-08",
            "2013-10-14",
            "2014-10-13",
            "2015-10-12",
            "2016-10-10",
            "2017-10-09",
            "2018-10-08",
            "2019-10-14",
            "2020-10-12",
            "2021-10-11",
            "2022-10-10",
            "2023-10-09",
        )
        for subdiv, change_year in (
            ("AK", 2015),
            ("DC", 2019),
            ("ME", 2019),
            ("NM", 2019),
            ("NE", 2020),
            ("VA", 2020),
        ):
            self.assertHolidayName(name_1, self.state_hols[subdiv], range(1971, change_year))
            self.assertHolidayName(name_2, self.state_hols[subdiv], range(change_year, 2050))
            self.assertHoliday(self.state_hols[subdiv], dts)
            self.assertNoHolidayName(
                name_1, self.state_hols[subdiv], range(1865, 1971), range(change_year, 2050)
            )
            self.assertNoHolidayName(name_2, self.state_hols[subdiv], range(1865, change_year))

        vi_name = "Columbus Day and Puerto Rico Friendship Day"
        vi_holidays = self.state_hols["VI"]
        self.assertHolidayName(vi_name, vi_holidays, dts)
        self.assertHolidayName(vi_name, vi_holidays, range(1937, 2050))
        self.assertNoHolidayName(vi_name, vi_holidays, range(1865, 1937))

    def test_columbus_day_al(self):
        name_1 = "Columbus Day / Fraternal Day"
        name_2 = "Columbus Day / American Indian Heritage Day / Fraternal Day"
        al_holidays = self.state_hols["AL"]
        dts = (
            "2010-10-11",
            "2011-10-10",
            "2012-10-08",
            "2013-10-14",
            "2014-10-13",
            "2015-10-12",
            "2016-10-10",
            "2017-10-09",
            "2018-10-08",
            "2019-10-14",
            "2020-10-12",
            "2021-10-11",
            "2022-10-10",
            "2023-10-09",
        )
        self.assertHolidayName(name_1, al_holidays, range(1971, 2000))
        self.assertHolidayName(name_2, al_holidays, range(2000, 2050))
        self.assertHoliday(al_holidays, dts)
        self.assertNoHolidayName(name_1, al_holidays, range(1865, 1970))
        self.assertNoHolidayName(name_2, al_holidays, range(1865, 2000))

    def test_columbus_day_ca(self):
        name = "Columbus Day"
        ca_holidays = self.state_hols["CA"]
        dts = (
            "1990-10-08",
            "1995-10-09",
            "2000-10-09",
            "2001-10-08",
            "2002-10-14",
            "2003-10-13",
            "2004-10-11",
            "2005-10-10",
            "2006-10-09",
            "2007-10-08",
            "2008-10-13",
        )
        self.assertHolidayName(name, ca_holidays, range(1971, 2009))
        self.assertHoliday(ca_holidays, dts)
        self.assertNoHolidayName(name, ca_holidays, range(1865, 1970), range(2009, 2050))

    def test_columbus_day_ri(self):
        name_1 = "Columbus Day"
        name_2 = "Indigenous Peoples' Day / Columbus Day"
        ri_holidays = self.state_hols["RI"]
        dts = (
            "2010-10-11",
            "2011-10-10",
            "2012-10-08",
            "2013-10-14",
            "2014-10-13",
            "2015-10-12",
            "2016-10-10",
            "2017-10-09",
            "2018-10-08",
            "2019-10-14",
            "2020-10-12",
            "2021-10-11",
            "2022-10-10",
            "2023-10-09",
        )
        self.assertHolidayName(name_1, ri_holidays, range(1971, 2022))
        self.assertHolidayName(name_2, ri_holidays, range(2022, 2050))
        self.assertHoliday(ri_holidays, dts)
        self.assertNoHolidayName(name_1, ri_holidays, range(1865, 1970))
        self.assertNoHolidayName(name_2, ri_holidays, range(1865, 2022))

    def test_columbus_day_sd(self):
        name_1 = "Columbus Day"
        name_2 = "Native Americans' Day"
        sd_holidays = self.state_hols["SD"]
        dts = (
            "2010-10-11",
            "2011-10-10",
            "2012-10-08",
            "2013-10-14",
            "2014-10-13",
            "2015-10-12",
            "2016-10-10",
            "2017-10-09",
            "2018-10-08",
            "2019-10-14",
            "2020-10-12",
            "2021-10-11",
            "2022-10-10",
            "2023-10-09",
        )
        self.assertHolidayName(name_1, sd_holidays, range(1937, 1990))
        self.assertHolidayName(name_2, sd_holidays, range(1990, 2050))
        self.assertHolidayName(name_2, sd_holidays, dts)
        self.assertNoHolidayName(name_1, sd_holidays, range(1865, 1937))
        self.assertNoHolidayName(name_2, sd_holidays, range(1865, 1990))

    def test_epiphany(self):
        name = "Epiphany"
        self.assertNoHolidayName(name)
        self.assertHolidayName(
            name, self.state_hols["PR"], (f"{year}-01-06" for year in range(1865, 2050))
        )

    def test_three_kings_day(self):
        name = "Three Kings Day"
        self.assertNoHolidayName(name)
        self.assertHolidayName(
            name, self.state_hols["VI"], (f"{year}-01-06" for year in range(1865, 2050))
        )

    def test_lee_jackson_day(self):
        name = "Lee Jackson Day"
        self.assertNoHolidayName(name)
        va_holidays = self.state_hols["VA"]
        self.assertNoHolidayName(name, va_holidays, range(1865, 1889), range(2021, 2050))
        self.assertHolidayName(name, va_holidays, (f"{year}-01-19" for year in range(1889, 1983)))
        dt = (
            "1983-01-17",
            "1990-01-15",
            "1995-01-16",
            "1999-01-18",
            "2000-01-14",
            "2010-01-15",
            "2011-01-14",
            "2012-01-13",
            "2013-01-18",
            "2014-01-17",
            "2015-01-16",
            "2016-01-15",
            "2017-01-13",
            "2018-01-12",
            "2019-01-18",
            "2020-01-17",
        )
        self.assertHolidayName(name, va_holidays, dt)

    def test_inauguration_day(self):
        name = "Inauguration Day"
        self.assertNoHolidayName(name)
        years_1 = range(1861, 1937, 4)
        years_2 = range(1937, 2050, 4)
        years_no = set(range(1865, 2050)).difference(set(years_1)).difference(set(years_2))
        obs_dt = (
            "1877-03-05",
            "1917-03-05",
            "1957-01-21",
            "1985-01-21",
        )
        for subdiv in ("DC", "LA", "MD", "VA"):
            self.assertHolidayName(
                name, self.state_hols[subdiv], (f"{year}-03-04" for year in years_1)
            )
            self.assertHolidayName(
                name, self.state_hols[subdiv], (f"{year}-01-20" for year in years_2)
            )
            self.assertHolidayName(name, UnitedStates(subdiv=subdiv), "1789-03-04")
            self.assertNoHolidayName(name, UnitedStates(subdiv=subdiv), "1788-03-04")
            self.assertNoHolidayName(name, self.state_hols[subdiv], years_no)
            self.assertHolidayName(f"{name} (observed)", self.state_hols[subdiv], obs_dt)
            self.assertNoNonObservedHolidayName(
                f"{name} (observed)", UnitedStates(subdiv=subdiv, observed=False), obs_dt
            )

    def test_lincolns_birthday(self):
        name = "Lincoln's Birthday"
        self.assertNoHolidayName(name)
        obs_dt = (
            "2011-02-11",
            "2012-02-13",
            "2017-02-13",
            "2022-02-11",
            "2023-02-13",
        )
        for subdiv in ("CT", "IA", "IL", "NJ", "NY"):
            self.assertHolidayName(
                name, self.state_hols[subdiv], (f"{year}-02-12" for year in range(1971, 2050))
            )
            self.assertHolidayName(f"{name} (observed)", self.state_hols[subdiv], obs_dt)
            self.assertNoNonObservedHolidayName(
                f"{name} (observed)", UnitedStates(subdiv=subdiv, observed=False), obs_dt
            )
        obs_dt = (
            "1994-02-11",
            "1995-02-13",
            "2000-02-11",
            "2005-02-11",
            "2006-02-13",
        )
        ca_holidays = self.state_hols["CA"]
        self.assertHolidayName(name, ca_holidays, (f"{year}-02-12" for year in range(1971, 2010)))
        self.assertHolidayName(f"{name} (observed)", ca_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="CA", observed=False), obs_dt
        )

    def test_susan_b_anthony_day(self):
        name = "Susan B. Anthony Day"
        self.assertNoHolidayName(name)
        for subdiv, start_year in (
            ("CA", 2014),
            ("FL", 2011),
            ("NY", 2004),
            ("WI", 1976),
        ):
            self.assertHolidayName(
                name,
                self.state_hols[subdiv],
                (f"{year}-02-15" for year in range(start_year, 2050)),
            )
            self.assertNoHolidayName(name, self.state_hols[subdiv], range(1865, start_year))

    def test_mardi_gras(self):
        name = "Mardi Gras"
        self.assertNoHolidayName(name)
        self.assertNoHolidayName(name, UnitedStates(subdiv="LA", years=1856))
        dt = (
            "2010-02-16",
            "2011-03-08",
            "2012-02-21",
            "2013-02-12",
            "2014-03-04",
            "2015-02-17",
            "2016-02-09",
            "2017-02-28",
            "2018-02-13",
            "2019-03-05",
            "2020-02-25",
            "2021-02-16",
            "2022-03-01",
            "2023-02-21",
        )
        self.assertHolidayName(name, self.state_hols["LA"], dt)

    def test_guam_discovery_day(self):
        name = "Guam Discovery Day"
        self.assertNoHolidayName(name)
        gu_holidays = self.state_hols["GU"]
        dt = (
            "2010-03-01",
            "2011-03-07",
            "2012-03-05",
            "2013-03-04",
            "2014-03-03",
            "2015-03-02",
            "2016-03-07",
            "2017-03-06",
            "2018-03-05",
            "2019-03-04",
            "2020-03-02",
            "2021-03-01",
            "2022-03-07",
            "2023-03-06",
        )
        self.assertHolidayName(name, gu_holidays, dt)
        self.assertNoHolidayName(name, gu_holidays, range(1865, 1970))

    def test_casimir_pulaski_day(self):
        name = "Casimir Pulaski Day"
        self.assertNoHolidayName(name)
        il_holidays = self.state_hols["IL"]
        dt = (
            "2010-03-01",
            "2011-03-07",
            "2012-03-05",
            "2013-03-04",
            "2014-03-03",
            "2015-03-02",
            "2016-03-07",
            "2017-03-06",
            "2018-03-05",
            "2019-03-04",
            "2020-03-02",
            "2021-03-01",
            "2022-03-07",
            "2023-03-06",
        )
        self.assertHolidayName(name, il_holidays, dt)
        self.assertNoHolidayName(name, il_holidays, range(1865, 1978))

    def test_texas_independence_day(self):
        name = "Texas Independence Day"
        self.assertNoHolidayName(name)
        tx_holidays = self.state_hols["TX"]
        self.assertHolidayName(name, tx_holidays, (f"{year}-03-02" for year in range(1874, 2050)))
        self.assertNoHolidayName(name, tx_holidays, range(1865, 1874))

    def test_town_meeting_day(self):
        name = "Town Meeting Day"
        self.assertNoHolidayName(name)
        self.assertNoHolidayName(name, UnitedStates(subdiv="VT", years=1799))
        dt = (
            "2010-03-02",
            "2011-03-01",
            "2012-03-06",
            "2013-03-05",
            "2014-03-04",
            "2015-03-03",
            "2016-03-01",
            "2017-03-07",
            "2018-03-06",
            "2019-03-05",
            "2020-03-03",
            "2021-03-02",
            "2022-03-01",
            "2023-03-07",
        )
        self.assertHolidayName(name, self.state_hols["VT"], dt)

    def test_evacuation_day(self):
        name = "Evacuation Day"
        self.assertNoHolidayName(name)
        ma_holidays = self.state_hols["MA"]
        self.assertHolidayName(name, ma_holidays, (f"{year}-03-17" for year in range(1901, 2050)))
        self.assertNoHolidayName(name, ma_holidays, range(1865, 1901))
        obs_dt = (
            "2012-03-19",
            "2013-03-18",
            "2018-03-19",
            "2019-03-18",
        )
        self.assertHolidayName(f"{name} (observed)", ma_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="MA", observed=False), obs_dt
        )

    def test_emancipation_day_in_puerto_rico(self):
        name = "Emancipation Day"
        self.assertNoHolidayName(name)
        pr_holidays = self.state_hols["PR"]
        self.assertHolidayName(name, pr_holidays, (f"{year}-03-22" for year in range(1865, 2050)))
        obs_dt = (
            "1998-03-23",
            "2009-03-23",
            "2015-03-23",
            "2020-03-23",
        )
        self.assertHolidayName(f"{name} (observed)", pr_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="PR", observed=False), obs_dt
        )

    def test_commonwealth_covenant_day(self):
        name = "Commonwealth Covenant Day"
        self.assertNoHolidayName(name)
        mp_holidays = self.state_hols["MP"]
        self.assertHolidayName(name, mp_holidays, (f"{year}-03-24" for year in range(1865, 2050)))
        obs_dt = (
            "2012-03-23",
            "2013-03-25",
            "2018-03-23",
            "2019-03-25",
        )
        self.assertHolidayName(f"{name} (observed)", mp_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="MP", observed=False), obs_dt
        )

    def test_prince_jonah_kuhio_kalanianaole_day(self):
        name = "Prince Jonah Kuhio Kalanianaole Day"
        self.assertNoHolidayName(name)
        hi_holidays = self.state_hols["HI"]
        self.assertHolidayName(name, hi_holidays, (f"{year}-03-26" for year in range(1949, 2050)))
        self.assertNoHolidayName(name, hi_holidays, range(1865, 1949))
        obs_dt = (
            "2011-03-25",
            "2016-03-25",
            "2017-03-27",
            "2022-03-25",
            "2023-03-27",
        )
        self.assertHolidayName(f"{name} (observed)", hi_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="HI", observed=False), obs_dt
        )

    def test_sewards_day(self):
        name = "Seward's Day"
        self.assertNoHolidayName(name)
        ak_holidays = self.state_hols["AK"]
        self.assertHolidayName(
            name,
            ak_holidays,
            (f"{year}-03-30" for year in (*range(1918, 1921), *range(1922, 1955))),
        )
        self.assertNoHolidayName(name, ak_holidays, range(1865, 1918), 1921)
        dt = (
            "1955-03-28",
            "2010-03-29",
            "2011-03-28",
            "2012-03-26",
            "2013-03-25",
            "2014-03-31",
            "2015-03-30",
            "2016-03-28",
            "2017-03-27",
            "2018-03-26",
            "2019-03-25",
            "2020-03-30",
            "2021-03-29",
            "2022-03-28",
            "2023-03-27",
        )
        self.assertHolidayName(name, ak_holidays, dt)

    def test_cesar_chavez_day(self):
        name = "Cesar Chavez Day"
        self.assertNoHolidayName(name)
        for subdiv, start_year in (
            ("CA", 1995),
            ("CO", 2001),
            ("TX", 2000),
        ):
            self.assertHolidayName(
                name,
                self.state_hols[subdiv],
                (f"{year}-03-31" for year in range(start_year, 2050)),
            )
            self.assertNoHolidayName(name, self.state_hols[subdiv], range(1865, start_year))

        obs_dt = (
            "1996-04-01",
            "2002-04-01",
            "2013-04-01",
            "2019-04-01",
        )
        self.assertHolidayName(f"{name} (observed)", self.state_hols["CA"], obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="CA", observed=False), obs_dt
        )

    def test_transfer_day(self):
        name = "Transfer Day"
        self.assertNoHolidayName(name)
        self.assertHolidayName(
            name, self.state_hols["VI"], (f"{year}-03-31" for year in range(1865, 2050))
        )

    def test_emancipation_day(self):
        name = "Emancipation Day"
        self.assertNoHolidayName(name)
        dc_holidays = self.state_hols["DC"]
        self.assertHolidayName(name, dc_holidays, (f"{year}-04-16" for year in range(2005, 2050)))
        self.assertNoHolidayName(name, dc_holidays, range(1865, 2005))
        obs_dt = (
            "2011-04-15",
            "2016-04-15",
            "2017-04-17",
            "2022-04-15",
            "2023-04-17",
        )
        self.assertHolidayName(f"{name} (observed)", dc_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="DC", observed=False), obs_dt
        )

    def test_american_samoa_flag_day(self):
        name = "American Samoa Flag Day"
        self.assertNoHolidayName(name)
        as_holidays = self.state_hols["AS"]
        self.assertHolidayName(name, as_holidays, (f"{year}-04-17" for year in range(1901, 2050)))
        self.assertNoHolidayName(name, as_holidays, range(1865, 1901))
        obs_dt = (
            "2004-04-16",
            "2005-04-18",
            "2010-04-16",
            "2011-04-18",
            "2016-04-18",
            "2021-04-16",
            "2022-04-18",
        )
        self.assertHolidayName(f"{name} (observed)", as_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="AS", observed=False), obs_dt
        )

    def test_patriots_day(self):
        name = "Patriots' Day"
        self.assertNoHolidayName(name)
        dt = (
            "1969-04-21",
            "2010-04-19",
            "2011-04-18",
            "2012-04-16",
            "2013-04-15",
            "2014-04-21",
            "2015-04-20",
            "2016-04-18",
            "2017-04-17",
            "2018-04-16",
            "2019-04-15",
            "2020-04-20",
            "2021-04-19",
            "2022-04-18",
            "2023-04-17",
        )
        for subdiv in ("MA", "ME"):
            self.assertHolidayName(
                name, self.state_hols[subdiv], (f"{year}-04-19" for year in range(1894, 1969))
            )
            self.assertNoHolidayName(name, self.state_hols[subdiv], range(1865, 1894))
            self.assertHolidayName(name, self.state_hols[subdiv], dt)

    def test_holy_thursday(self):
        name = "Holy Thursday"
        self.assertNoHolidayName(name)
        vi_holidays = self.state_hols["VI"]
        self.assertHolidayName(name, vi_holidays, range(1865, 2050))
        dt = (
            "2010-04-01",
            "2011-04-21",
            "2012-04-05",
            "2013-03-28",
            "2014-04-17",
            "2015-04-02",
            "2016-03-24",
            "2017-04-13",
            "2018-03-29",
            "2019-04-18",
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
        )
        self.assertHolidayName(name, vi_holidays, dt)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertNoHolidayName(name)
        dt = (
            "2010-04-02",
            "2011-04-22",
            "2012-04-06",
            "2013-03-29",
            "2014-04-18",
            "2015-04-03",
            "2016-03-25",
            "2017-04-14",
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
        )
        for subdiv in (
            "CT",
            "DE",
            "GU",
            "IN",
            "KY",
            "LA",
            "MP",
            "NC",
            "NJ",
            "PR",
            "TN",
            "TX",
            "VI",
        ):
            self.assertHolidayName(name, self.state_hols[subdiv], range(1865, 2050))
            self.assertHolidayName(name, self.state_hols[subdiv], dt)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertNoHolidayName(name)
        vi_holidays = self.state_hols["VI"]
        self.assertHolidayName(name, vi_holidays, range(1865, 2050))
        dt = (
            "2010-04-05",
            "2011-04-25",
            "2012-04-09",
            "2013-04-01",
            "2014-04-21",
            "2015-04-06",
            "2016-03-28",
            "2017-04-17",
            "2018-04-02",
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
        )
        self.assertHolidayName(name, vi_holidays, dt)

    def test_confederate_memorial_day(self):
        name = "Confederate Memorial Day"
        self.assertNoHolidayName(name)
        dt = (
            "2010-04-26",
            "2011-04-25",
            "2012-04-23",
            "2013-04-22",
            "2014-04-28",
            "2015-04-27",
            "2016-04-25",
            "2017-04-24",
            "2018-04-23",
            "2019-04-22",
            "2020-04-27",
            "2021-04-26",
            "2022-04-25",
            "2023-04-24",
        )
        for subdiv in ("AL", "SC"):
            self.assertHolidayName(name, self.state_hols[subdiv], range(1866, 2050))
            self.assertNoHolidayName(name, self.state_hols[subdiv], range(1865, 1866))
            self.assertHolidayName(name, self.state_hols[subdiv], dt)

        ms_holidays = self.state_hols["MS"]
        self.assertHolidayName(
            name,
            ms_holidays,
            "2010-04-26",
            "2011-04-25",
            "2012-04-30",
            "2013-04-29",
            "2014-04-28",
            "2015-04-27",
            "2016-04-25",
            "2017-04-24",
            "2018-04-30",
            "2019-04-29",
            "2020-04-27",
            "2021-04-26",
            "2022-04-25",
            "2023-04-24",
            "2024-04-29",
        )
        self.assertHolidayName(name, ms_holidays, range(1866, 2050))
        self.assertNoHolidayName(name, ms_holidays, range(1865, 1866))

        tx_holidays = self.state_hols["TX"]
        self.assertHolidayName(name, tx_holidays, (f"{year}-01-19" for year in range(1931, 2050)))
        self.assertNoHolidayName(name, tx_holidays, range(1865, 1931))

        ga_holidays = self.state_hols["GA"]
        self.assertHolidayName(name, ga_holidays, range(1866, 2016))
        self.assertNoHolidayName(name, ga_holidays, range(2016, 2050))
        self.assertHolidayName(
            name,
            ga_holidays,
            "2010-04-26",
            "2011-04-25",
            "2012-04-23",
            "2013-04-22",
            "2014-04-28",
            "2015-04-27",
        )

        name = "State Holiday"
        self.assertNoHolidayName(name)
        self.assertHolidayName(name, ga_holidays, range(2016, 2050))
        self.assertHolidayName(
            name,
            ga_holidays,
            "2016-04-25",
            "2017-04-24",
            "2018-04-23",
            "2019-04-22",
            "2020-04-10",
            "2021-04-26",
            "2022-04-25",
            "2023-04-24",
        )

    def test_san_jacinto_day(self):
        name = "San Jacinto Day"
        self.assertNoHolidayName(name)
        tx_holidays = self.state_hols["TX"]
        self.assertHolidayName(name, tx_holidays, (f"{year}-04-21" for year in range(1875, 2050)))
        self.assertNoHolidayName(name, tx_holidays, range(1865, 1875))

    def test_arbor_day(self):
        name = "Arbor Day"
        self.assertNoHolidayName(name)
        ne_holidays = self.state_hols["NE"]
        dt = (
            "1989-04-28",
            "2010-04-30",
            "2011-04-29",
            "2012-04-27",
            "2013-04-26",
            "2014-04-25",
            "2015-04-24",
            "2016-04-29",
            "2017-04-28",
            "2018-04-27",
            "2019-04-26",
            "2020-04-24",
            "2021-04-30",
            "2022-04-29",
            "2023-04-28",
        )
        self.assertHolidayName(name, ne_holidays, (f"{year}-04-22" for year in range(1875, 1989)))
        self.assertNoHolidayName(name, ne_holidays, range(1865, 1875))
        self.assertHolidayName(name, ne_holidays, dt)

    def test_primary_election_day(self):
        name = "Primary Election Day"
        self.assertNoHolidayName(name)
        in_holidays = self.state_hols["IN"]
        dt = (
            "2006-05-02",
            "2008-05-06",
            "2010-05-04",
            "2012-05-08",
            "2014-05-06",
            "2015-05-05",
            "2016-05-03",
            "2017-05-02",
            "2018-05-08",
            "2019-05-07",
            "2020-05-05",
            "2021-05-04",
            "2022-05-03",
            "2023-05-02",
        )
        self.assertHolidayName(name, in_holidays, 2006, 2008, 2010, 2012, 2014, range(2015, 2050))
        self.assertNoHolidayName(name, in_holidays, range(1865, 2006), 2007, 2009, 2011, 2013)
        self.assertHolidayName(name, in_holidays, dt)

    def test_truman_day(self):
        name = "Truman Day"
        self.assertNoHolidayName(name)
        mo_holidays = self.state_hols["MO"]
        self.assertHolidayName(name, mo_holidays, (f"{year}-05-08" for year in range(1949, 2050)))
        self.assertNoHolidayName(name, mo_holidays, range(1865, 1949))
        obs_dt = (
            "2010-05-07",
            "2011-05-09",
            "2016-05-09",
            "2021-05-07",
            "2022-05-09",
        )
        self.assertHolidayName(f"{name} (observed)", mo_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="MO", observed=False), obs_dt
        )

    def test_jefferson_davis_birthday(self):
        name = "Jefferson Davis Birthday"
        self.assertNoHolidayName(name)
        al_holidays = self.state_hols["AL"]
        self.assertHolidayName(name, al_holidays, range(1890, 2050))
        self.assertNoHolidayName(name, al_holidays, range(1865, 1890))
        dt = (
            "2010-06-07",
            "2011-06-06",
            "2012-06-04",
            "2013-06-03",
            "2014-06-02",
            "2015-06-01",
            "2016-06-06",
            "2017-06-05",
            "2018-06-04",
            "2019-06-03",
            "2020-06-01",
            "2021-06-07",
            "2022-06-06",
            "2023-06-05",
        )
        self.assertHolidayName(name, al_holidays, dt)

    def test_kamehameha_day(self):
        name = "Kamehameha Day"
        self.assertNoHolidayName(name)
        hi_holidays = self.state_hols["HI"]
        self.assertHolidayName(name, hi_holidays, (f"{year}-06-11" for year in range(1872, 2050)))
        self.assertNoHolidayName(name, hi_holidays, range(1865, 1872))
        obs_dt = (
            "2011-06-10",
            "2016-06-10",
            "2017-06-12",
            "2022-06-10",
            "2023-06-12",
        )
        self.assertHolidayName(f"{name} (observed)", hi_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="HI", observed=False), obs_dt
        )
        self.assertNoHolidayName(f"{name} (observed)", hi_holidays, range(1872, 2011))

    def test_emancipation_day_in_texas(self):
        name = "Emancipation Day In Texas"
        self.assertNoHolidayName(name)
        tx_holidays = self.state_hols["TX"]
        self.assertHolidayName(name, tx_holidays, (f"{year}-06-19" for year in range(1980, 2050)))
        self.assertNoHolidayName(name, tx_holidays, range(1865, 1980))

    def test_west_virginia_day(self):
        name = "West Virginia Day"
        self.assertNoHolidayName(name)
        wv_holidays = self.state_hols["WV"]
        self.assertHolidayName(name, wv_holidays, (f"{year}-06-20" for year in range(1927, 2050)))
        self.assertNoHolidayName(name, wv_holidays, range(1865, 1927))
        obs_dt = (
            "2010-06-21",
            "2015-06-19",
            "2020-06-19",
            "2021-06-21",
        )
        self.assertHolidayName(f"{name} (observed)", wv_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="WV", observed=False), obs_dt
        )

    def test_emancipation_day_in_virgin_islands(self):
        name = "Emancipation Day"
        self.assertNoHolidayName(name)
        self.assertHolidayName(
            name, self.state_hols["VI"], (f"{year}-07-03" for year in range(1865, 2050))
        )

    def test_manua_islands_cession_day(self):
        name = "Manu'a Islands Cession Day"
        self.assertNoHolidayName(name)
        as_holidays = self.state_hols["AS"]
        self.assertHolidayName(name, as_holidays, (f"{year}-07-16" for year in range(1983, 2050)))
        self.assertNoHolidayName(name, as_holidays, range(1865, 1983))
        obs_dt = (
            "2000-07-17",
            "2005-07-15",
            "2006-07-17",
            "2011-07-15",
            "2016-07-15",
            "2017-07-17",
            "2022-07-15",
            "2023-07-17",
        )
        self.assertHolidayName(f"{name} (observed)", as_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="AS", observed=False), obs_dt
        )

    def test_liberation_day_guam(self):
        name = "Liberation Day (Guam)"
        self.assertNoHolidayName(name)
        gu_holidays = self.state_hols["GU"]
        self.assertHolidayName(name, gu_holidays, (f"{year}-07-21" for year in range(1945, 2050)))
        self.assertNoHolidayName(name, gu_holidays, range(1865, 1945))

    def test_pioneer_day(self):
        name = "Pioneer Day"
        self.assertNoHolidayName(name)
        ut_holidays = self.state_hols["UT"]
        self.assertHolidayName(name, ut_holidays, (f"{year}-07-24" for year in range(1865, 2050)))
        self.assertNoHolidayName(name, UnitedStates(subdiv="UT", years=1848))
        obs_dt = (
            "2010-07-23",
            "2011-07-25",
            "2016-07-25",
            "2021-07-23",
            "2022-07-25",
        )
        self.assertHolidayName(f"{name} (observed)", ut_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="UT", observed=False), obs_dt
        )

    def test_constitution_day(self):
        name = "Constitution Day"
        self.assertNoHolidayName(name)
        pr_holidays = self.state_hols["PR"]
        self.assertHolidayName(name, pr_holidays, (f"{year}-07-25" for year in range(1865, 2050)))
        obs_dt = (
            "1999-07-26",
            "2004-07-26",
            "2010-07-26",
            "2021-07-26",
        )
        self.assertHolidayName(f"{name} (observed)", pr_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="PR", observed=False), obs_dt
        )

    def test_victory_day(self):
        name = "Victory Day"
        self.assertNoHolidayName(name)
        ri_holidays = self.state_hols["RI"]
        self.assertHolidayName(name, ri_holidays, range(1948, 2050))
        self.assertNoHolidayName(name, ri_holidays, range(1865, 1948))
        dt = (
            "2010-08-09",
            "2011-08-08",
            "2012-08-13",
            "2013-08-12",
            "2014-08-11",
            "2015-08-10",
            "2016-08-08",
            "2017-08-14",
            "2018-08-13",
            "2019-08-12",
            "2020-08-10",
            "2021-08-09",
            "2022-08-08",
            "2023-08-14",
        )
        self.assertHolidayName(name, ri_holidays, dt)

    def test_statehood_day(self):
        name = "Statehood Day"
        self.assertNoHolidayName(name)
        hi_holidays = self.state_hols["HI"]
        self.assertHolidayName(name, hi_holidays, range(1959, 2050))
        self.assertNoHolidayName(name, hi_holidays, range(1865, 1959))
        dt = (
            "2010-08-20",
            "2011-08-19",
            "2012-08-17",
            "2013-08-16",
            "2014-08-15",
            "2015-08-21",
            "2016-08-19",
            "2017-08-18",
            "2018-08-17",
            "2019-08-16",
            "2020-08-21",
            "2021-08-20",
            "2022-08-19",
            "2023-08-18",
        )
        self.assertHolidayName(name, hi_holidays, dt)

    def test_bennington_battle_day(self):
        name = "Bennington Battle Day"
        self.assertNoHolidayName(name)
        vt_holidays = self.state_hols["VT"]
        self.assertHolidayName(name, vt_holidays, (f"{year}-08-16" for year in range(1865, 2050)))
        self.assertNoHolidayName(name, UnitedStates(subdiv="VT", years=1777))
        obs_dt = (
            "2009-08-17",
            "2014-08-15",
            "2015-08-17",
            "2020-08-17",
        )
        self.assertHolidayName(f"{name} (observed)", vt_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="VT", observed=False), obs_dt
        )

    def test_lyndon_baines_johnson_day(self):
        name = "Lyndon Baines Johnson Day"
        self.assertNoHolidayName(name)
        tx_holidays = self.state_hols["TX"]
        self.assertHolidayName(name, tx_holidays, (f"{year}-08-27" for year in range(1973, 2050)))
        self.assertNoHolidayName(name, tx_holidays, range(1865, 1973))

    def test_commonwealth_cultural_day(self):
        name = "Commonwealth Cultural Day"
        self.assertNoHolidayName(name)
        dt = (
            "2010-10-11",
            "2011-10-10",
            "2012-10-08",
            "2013-10-14",
            "2014-10-13",
            "2015-10-12",
            "2016-10-10",
            "2017-10-09",
            "2018-10-08",
            "2019-10-14",
            "2020-10-12",
            "2021-10-11",
            "2022-10-10",
            "2023-10-09",
        )
        self.assertHolidayName(name, self.state_hols["MP"], dt)

    def test_white_sunday(self):
        name = "White Sunday"
        self.assertNoHolidayName(name)
        as_holidays = self.state_hols["AS"]
        self.assertHolidayName(name, as_holidays, range(1865, 2050))
        dt = (
            "2010-10-10",
            "2011-10-09",
            "2012-10-14",
            "2013-10-13",
            "2014-10-12",
            "2015-10-11",
            "2016-10-09",
            "2017-10-08",
            "2018-10-14",
            "2019-10-13",
            "2020-10-11",
            "2021-10-10",
            "2022-10-09",
            "2023-10-08",
            "2024-10-13",
        )
        self.assertHolidayName(name, as_holidays, dt)

    def test_alaska_day(self):
        name = "Alaska Day"
        self.assertNoHolidayName(name)
        ak_holidays = self.state_hols["AK"]
        self.assertHolidayName(name, ak_holidays, (f"{year}-10-18" for year in range(1917, 2050)))
        self.assertNoHolidayName(name, ak_holidays, range(1865, 1917))
        obs_dt = (
            "2009-10-19",
            "2014-10-17",
            "2015-10-19",
            "2020-10-19",
        )
        self.assertHolidayName(f"{name} (observed)", ak_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="AK", observed=False), obs_dt
        )

    def test_nevada_day(self):
        name = "Nevada Day"
        self.assertNoHolidayName(name)
        nv_holidays = self.state_hols["NV"]
        self.assertHolidayName(name, nv_holidays, (f"{year}-10-31" for year in range(1933, 2000)))
        self.assertNoHolidayName(name, nv_holidays, range(1865, 1933))
        dt = (
            "2000-10-27",
            "2010-10-29",
            "2011-10-28",
            "2012-10-26",
            "2013-10-25",
            "2014-10-31",
            "2015-10-30",
            "2016-10-28",
            "2017-10-27",
            "2018-10-26",
            "2019-10-25",
            "2020-10-30",
            "2021-10-29",
            "2022-10-28",
            "2023-10-27",
        )
        self.assertHolidayName(name, nv_holidays, dt)

        obs_dt = (
            "1992-10-30",
            "1993-11-01",
            "1998-10-30",
            "1999-11-01",
        )
        self.assertHolidayName(f"{name} (observed)", nv_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="NV", observed=False), obs_dt
        )

    def test_liberty_day(self):
        name = "Liberty Day"
        self.assertNoHolidayName(name)
        self.assertHolidayName(
            name, self.state_hols["VI"], (f"{year}-11-01" for year in range(1865, 2050))
        )

    def test_election_day(self):
        name = "Election Day"
        self.assertNoHolidayName(name)
        dt = (
            "2008-11-04",
            "2010-11-02",
            "2012-11-06",
            "2014-11-04",
            "2016-11-08",
            "2018-11-06",
            "2020-11-03",
            "2022-11-08",
        )
        for subdiv in ("DE", "HI", "IL", "LA", "MI", "MP", "MT", "NH", "NJ", "WV"):
            self.assertHolidayName(name, self.state_hols[subdiv], dt)
            self.assertNoHolidayName(
                name, self.state_hols[subdiv], range(1865, 2008), range(2009, 2050, 2)
            )

        dt = (
            "2008-11-04",
            "2010-11-02",
            "2012-11-06",
            "2014-11-04",
            "2015-11-03",
            "2016-11-08",
            "2017-11-07",
            "2018-11-06",
            "2019-11-05",
            "2020-11-03",
            "2021-11-02",
            "2022-11-08",
            "2023-11-07",
        )
        for subdiv in ("IN", "NY"):
            self.assertHolidayName(name, self.state_hols[subdiv], dt)
            self.assertNoHolidayName(name, self.state_hols[subdiv], range(1865, 2008))

        # For NON_OFFICIAL category
        # This is actually for the Presidential Election, but let's keep the same name
        # to prevent duplicates for states which have them as proper public holidays.

        dt = (
            "1868-11-03",
            "1872-11-05",
            "1876-11-07",
            "1880-11-02",
            "1884-11-04",
            "1888-11-06",
            "1892-11-08",
            "1896-11-03",
            "1900-11-06",
            "1904-11-08",
            "1908-11-03",
            "1912-11-05",
            "1916-11-07",
            "1920-11-02",
            "1924-11-04",
            "1928-11-06",
            "1932-11-08",
            "1936-11-03",
            "1940-11-05",
            "1944-11-07",
            "1948-11-02",
            "1952-11-04",
            "1956-11-06",
            "1960-11-08",
            "1964-11-03",
            "1968-11-05",
            "1972-11-07",
            "1976-11-02",
            "1980-11-04",
            "1984-11-06",
            "1988-11-08",
            "1992-11-03",
            "1996-11-05",
            "2000-11-07",
            "2004-11-02",
            "2008-11-04",
            "2012-11-06",
            "2016-11-08",
            "2020-11-03",
            "2024-11-05",
        )
        for subdiv in UnitedStates.subdivisions:
            holidays = UnitedStates(subdiv=subdiv, categories=UNOFFICIAL, years=range(1865, 2050))
            if subdiv in {"AS", "GU", "MP", "PR", "UM", "VI"}:
                self.assertNoHolidayName(name, holidays)
            else:
                self.assertNoHolidayName(
                    name, UnitedStates(subdiv=subdiv, categories=UNOFFICIAL, years=(1844, 1847))
                )
                self.assertHolidayName(name, holidays, dt)

    def test_valentines_day(self):
        name = "Valentine's Day"
        self.assertNoHolidayName(name)
        self.assertNoHolidayName(name, UnitedStates(categories=UNOFFICIAL, years=1846))
        self.assertHolidayName(
            name, self.unofficial_hols, (f"{year}-02-14" for year in range(1865, 2050))
        )

    def test_st_patricks_day(self):
        name = "Saint Patrick's Day"
        self.assertNoHolidayName(name)
        self.assertHolidayName(
            name, self.unofficial_hols, (f"{year}-03-17" for year in range(1865, 2050))
        )

    def test_halloween(self):
        name = "Halloween"
        self.assertNoHolidayName(name)
        self.assertHolidayName(
            name, self.unofficial_hols, (f"{year}-10-31" for year in range(1865, 2050))
        )

    def test_groundhog_day(self):
        name = "Groundhog Day"
        self.assertNoHolidayName(name)
        self.assertHolidayName(
            name, self.unofficial_hols, (f"{year}-02-02" for year in range(1886, 2050))
        )

    def test_all_souls_day(self):
        name = "All Souls' Day"
        self.assertNoHolidayName(name)
        self.assertHolidayName(
            name, self.state_hols["GU"], (f"{year}-11-02" for year in range(1865, 2050))
        )

    def test_citizenship_day(self):
        name = "Citizenship Day"
        self.assertNoHolidayName(name)
        mp_holidays = self.state_hols["MP"]
        self.assertHolidayName(name, mp_holidays, (f"{year}-11-04" for year in range(1865, 2050)))
        obs_dt = (
            "2012-11-05",
            "2017-11-03",
            "2018-11-05",
            "2023-11-03",
        )
        self.assertHolidayName(f"{name} (observed)", mp_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="MP", observed=False), obs_dt
        )

    def test_discovery_day(self):
        name = "Discovery Day"
        self.assertNoHolidayName(name)
        pr_holidays = self.state_hols["PR"]
        self.assertHolidayName(name, pr_holidays, (f"{year}-11-19" for year in range(1865, 2050)))
        obs_dt = (
            "2000-11-20",
            "2006-11-20",
            "2017-11-20",
            "2023-11-20",
        )
        self.assertHolidayName(f"{name} (observed)", pr_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="PR", observed=False), obs_dt
        )

    def test_day_after_thanksgiving(self):
        dt = (
            "2017-11-24",
            "2018-11-23",
            "2019-11-29",
            "2020-11-27",
            "2021-11-26",
            "2022-11-25",
            "2023-11-24",
            "2024-11-29",
        )
        for subdiv, name, start_year in (
            ("CA", "Day After Thanksgiving", 1975),
            ("DE", "Day After Thanksgiving", 1975),
            ("FL", "Friday After Thanksgiving", 1975),
            ("IN", "Lincoln's Birthday", 2010),
            ("MD", "American Indian Heritage Day", 2008),
            ("MI", "Day After Thanksgiving", 2017),
            ("NC", "Day After Thanksgiving", 1975),
            ("NH", "Day After Thanksgiving", 1975),
            ("NM", "Presidents' Day", None),
            ("NV", "Family Day", None),
            ("OK", "Day After Thanksgiving", 1975),
            ("PA", "Day After Thanksgiving", None),
            ("TX", "Friday After Thanksgiving", 1975),
            ("WV", "Day After Thanksgiving", 1975),
        ):
            self.assertNoHolidayName(name)
            self.assertHolidayName(name, self.state_hols[subdiv], dt)
            self.assertHolidayName(name, self.state_hols[subdiv], range(start_year or 1865, 2050))
            if start_year:
                self.assertNoHolidayName(name, self.state_hols[subdiv], range(1865, start_year))

    def test_robert_lee_birthday(self):
        name_1 = "Robert E. Lee's Birthday"
        name_2 = "State Holiday"
        ga_holidays = self.state_hols["GA"]
        self.assertNoHolidayName(name_1)
        self.assertNoHolidayName(name_2)
        self.assertHolidayName(name_1, ga_holidays, range(1986, 2016))
        self.assertHolidayName(name_2, ga_holidays, range(2016, 2050))
        self.assertNoHolidayName(name_1, ga_holidays, range(1865, 1986), range(2016, 2050))
        self.assertNoHolidayName(name_2, ga_holidays, range(1865, 2016))
        self.assertHolidayName(
            name_1,
            ga_holidays,
            "2010-11-26",
            "2011-11-25",
            "2012-11-23",
            "2013-11-29",
            "2014-11-28",
            "2015-11-27",
        )
        self.assertHolidayName(
            name_2,
            ga_holidays,
            "2016-11-25",
            "2017-11-24",
            "2018-11-23",
            "2019-11-29",
            "2020-11-27",
            "2021-11-26",
            "2022-11-25",
            "2023-11-24",
        )

    def test_lady_of_camarin_day(self):
        name = "Lady of Camarin Day"
        self.assertNoHolidayName(name)
        self.assertHolidayName(
            name, self.state_hols["GU"], (f"{year}-12-08" for year in range(1865, 2050))
        )

    def test_constitution_day_mp(self):
        name = "Constitution Day"
        self.assertNoHolidayName(name)
        mp_holidays = self.state_hols["MP"]
        self.assertHolidayName(name, mp_holidays, (f"{year}-12-08" for year in range(1865, 2050)))
        obs_dt = (
            "2012-12-07",
            "2013-12-09",
            "2018-12-07",
            "2019-12-09",
        )
        self.assertHolidayName(f"{name} (observed)", mp_holidays, obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="MP", observed=False), obs_dt
        )

    def test_christmas_eve(self):
        name = "Christmas Eve"
        self.assertNoHolidayName(name)
        obs_dt = (
            "2016-12-23",
            "2017-12-22",
            "2021-12-23",
            "2022-12-23",
            "2023-12-22",
        )
        for subdiv, start_year in (
            ("KS", 2013),
            ("MI", 2013),
            ("NC", 2013),
            ("TX", 1981),
            ("WI", 2012),
        ):
            self.assertHolidayName(
                name,
                self.state_hols[subdiv],
                (f"{year}-12-24" for year in range(start_year, 2050)),
            )
            self.assertNoHolidayName(name, self.state_hols[subdiv], range(1865, start_year))
            self.assertHolidayName(f"{name} (observed)", self.state_hols[subdiv], obs_dt)
            self.assertNoNonObservedHolidayName(
                f"{name} (observed)", UnitedStates(subdiv=subdiv, observed=False), obs_dt
            )

    def test_day_after_christmas(self):
        for subdiv, name, start_year in (
            ("NC", "Day After Christmas", 2013),
            ("TX", "Day After Christmas", 1981),
            ("VI", "Christmas Second Day", -1),
        ):
            self.assertNoHolidayName(name)
            self.assertHolidayName(
                name,
                self.state_hols[subdiv],
                (f"{year}-12-26" for year in range(start_year if start_year > 0 else 1865, 2050)),
            )
            if start_year > 0:
                self.assertNoHolidayName(name, self.state_hols[subdiv], range(1865, start_year))
        obs_dt = (
            "2015-12-28",
            "2016-12-27",
            "2020-12-28",
            "2021-12-27",
            "2022-12-27",
        )
        name = "Day After Christmas"
        self.assertHolidayName(f"{name} (observed)", self.state_hols["NC"], obs_dt)
        self.assertNoNonObservedHolidayName(
            f"{name} (observed)", UnitedStates(subdiv="NC", observed=False), obs_dt
        )

    def test_new_years_eve(self):
        name = "New Year's Eve"
        self.assertNoHolidayName(name)
        obs_dt = (
            "2016-12-30",
            "2022-12-30",
        )
        for subdiv, start_year in (
            ("KY", 2013),
            ("MI", 2013),
            ("WI", 2012),
        ):
            self.assertHolidayName(
                name,
                self.state_hols[subdiv],
                (f"{year}-12-31" for year in range(start_year, 2050)),
            )
            self.assertNoHolidayName(name, self.state_hols[subdiv], range(1865, start_year))
            self.assertHolidayName(f"{name} (observed)", self.state_hols[subdiv], obs_dt)
            self.assertNoNonObservedHolidayName(
                f"{name} (observed)", UnitedStates(subdiv=subdiv, observed=False), obs_dt
            )

    def test_frances_xavier_cabrini_day(self):
        name = "Frances Xavier Cabrini Day"
        self.assertNoHolidayName(name)
        co_holidays = self.state_hols["CO"]
        self.assertNoHolidayName(name, co_holidays, range(1865, 2019))
        self.assertHolidayName(name, co_holidays, range(2020, 2050))
        self.assertHoliday(
            co_holidays,
            (
                "2020-10-05",
                "2021-10-04",
                "2022-10-03",
                "2023-10-02",
                "2024-10-07",
            ),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import GOVERNMENT
from holidays.countries.georgia import Georgia, GE, GEO
from tests.common import CommonCountryTests


class TestGeorgia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Georgia)

    def test_country_aliases(self):
        self.assertAliases(Georgia, GE, GEO)

    def test_no_holidays(self):
        self.assertNoHolidays(Georgia(years=1990))

    def test_family_sanctity_day(self):
        self.assertHolidays(
            Georgia(categories=GOVERNMENT, years=2024),
            ("2024-05-17", "ოჯახის სიწმინდისა და მშობლების პატივისცემის დღე"),
        )

    def test_2020(self):
        # https://en.wikipedia.org/wiki/Public_holidays_in_Georgia_(country)
        self.assertHolidayDates(
            "2020-01-01",
            "2020-01-02",
            "2020-01-07",
            "2020-01-19",
            "2020-03-03",
            "2020-03-08",
            "2020-04-09",
            "2020-04-17",
            "2020-04-18",
            "2020-04-19",
            "2020-04-20",
            "2020-05-09",
            "2020-05-12",
            "2020-05-26",
            "2020-08-28",
            "2020-10-14",
            "2020-11-23",
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "ახალი წელი"),
            ("2022-01-02", "ახალი წელი"),
            ("2022-01-07", "ქრისტეშობა"),
            ("2022-01-19", "ნათლისღება"),
            ("2022-03-03", "დედის დღე"),
            ("2022-03-08", "ქალთა საერთაშორისო დღე"),
            ("2022-04-09", "ეროვნული ერთიანობის დღე"),
            ("2022-04-22", "წითელი პარასკევი"),
            ("2022-04-23", "დიდი შაბათი"),
            ("2022-04-24", "აღდგომა"),
            ("2022-04-25", "შავი ორშაბათი"),
            ("2022-05-09", "ფაშიზმზე გამარჯვების დღე"),
            ("2022-05-12", "წმინდა ანდრია პირველწოდებულის დღე"),
            ("2022-05-26", "დამოუკიდებლობის დღე"),
            ("2022-08-28", "მარიამობა"),
            ("2022-10-14", "მცხეთობის"),
            ("2022-11-23", "გიორგობა"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-02", "New Year's Day"),
            ("2022-01-07", "Christmas Day"),
            ("2022-01-19", "Epiphany"),
            ("2022-03-03", "Mother's Day"),
            ("2022-03-08", "International Women's Day"),
            ("2022-04-09", "National Unity Day"),
            ("2022-04-22", "Good Friday"),
            ("2022-04-23", "Holy Saturday"),
            ("2022-04-24", "Easter Sunday"),
            ("2022-04-25", "Easter Monday"),
            ("2022-05-09", "Day of Victory over Fascism"),
            ("2022-05-12", "Saint Andrew's Day"),
            ("2022-05-26", "Independence Day"),
            ("2022-08-28", "Dormition of the Mother of God"),
            ("2022-10-14", "Holiday of Svetitskhovloba, Robe of Jesus"),
            ("2022-11-23", "Saint George's Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-02", "Новий рік"),
            ("2022-01-07", "Різдво Христове"),
            ("2022-01-19", "Богоявлення"),
            ("2022-03-03", "День матері"),
            ("2022-03-08", "Міжнародний жіночий день"),
            ("2022-04-09", "День національної єдності"),
            ("2022-04-22", "Страсна пʼятниця"),
            ("2022-04-23", "Велика субота"),
            ("2022-04-24", "Великдень"),
            ("2022-04-25", "Великодній понеділок"),
            ("2022-05-09", "День перемоги над фашизмом"),
            ("2022-05-12", "День Святого Андрія Первозваного"),
            ("2022-05-26", "День незалежності"),
            ("2022-08-28", "Успіння Пресвятої Богородиці"),
            ("2022-10-14", "Свято Светіцховлоба, Ризи Господньої"),
            ("2022-11-23", "День Святого Георгія"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.bahrain import Bahrain, BH, BAH
from tests.common import CommonCountryTests


class TestBahrain(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Bahrain)

    def test_country_aliases(self):
        self.assertAliases(Bahrain, BH, BAH)

    def test_2022(self):
        self.assertHoliday(
            "2022-01-01",
            "2022-05-01",
            "2022-05-02",
            "2022-05-03",
            "2022-05-04",
            "2022-07-09",
            "2022-07-10",
            "2022-07-11",
            "2022-07-30",
            "2022-08-07",
            "2022-08-08",
            "2022-10-08",
            "2022-12-16",
            "2022-12-17",
        )

    def test_2023(self):
        self.assertHoliday(
            "2023-01-01",
            "2023-04-21",
            "2023-04-22",
            "2023-04-23",
            "2023-05-01",
            "2023-06-28",
            "2023-06-29",
            "2023-06-30",
            "2023-07-19",
            "2023-07-27",
            "2023-07-28",
            "2023-09-27",
            "2023-12-16",
            "2023-12-17",
        )

    def test_hijri_based(self):
        # Eid Al-Fitr.
        self.assertHoliday(
            "2020-05-24",
            "2020-05-25",
            "2020-05-26",
        )

        # Eid Al-Adha.
        self.assertHoliday(
            "2020-07-31",
            "2020-08-01",
            "2020-08-02",
        )

        # Islamic New Year.
        self.assertHoliday(
            "2008-01-10",
            "2008-12-29",
            "2020-08-20",
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "رأس السنة الميلادية"),
            ("2023-04-21", "(تقدير) عيد الفطر"),
            ("2023-04-22", "(تقدير) عطلة عيد الفطر"),
            ("2023-04-23", "(تقدير) عطلة عيد الفطر"),
            ("2023-05-01", "عيد العمال"),
            ("2023-06-28", "(تقدير) عيد الأضحى"),
            ("2023-06-29", "(تقدير) عطلة عيد الأضحى"),
            ("2023-06-30", "(تقدير) عطلة عيد الأضحى"),
            ("2023-07-19", "(تقدير) رأس السنة الهجرية"),
            ("2023-07-27", "(تقدير) ليلة عاشورة"),
            ("2023-07-28", "(تقدير) عاشورة"),
            ("2023-09-27", "(تقدير) عيد المولد النبوي"),
            ("2023-12-16", "اليوم الوطني"),
            ("2023-12-17", "اليوم الوطني"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-04-21", "Eid al-Fitr (estimated)"),
            ("2023-04-22", "Eid al-Fitr Holiday (estimated)"),
            ("2023-04-23", "Eid al-Fitr Holiday (estimated)"),
            ("2023-05-01", "Labor Day"),
            ("2023-06-28", "Eid al-Adha (estimated)"),
            ("2023-06-29", "Eid al-Adha Holiday (estimated)"),
            ("2023-06-30", "Eid al-Adha Holiday (estimated)"),
            ("2023-07-19", "Islamic New Year (estimated)"),
            ("2023-07-27", "Ashura Eve (estimated)"),
            ("2023-07-28", "Ashura (estimated)"),
            ("2023-09-27", "Prophet's Birthday (estimated)"),
            ("2023-12-16", "National Day"),
            ("2023-12-17", "National Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.north_macedonia import NorthMacedonia, MK, MKD
from tests.common import CommonCountryTests


class TestNorthMacedonia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(NorthMacedonia)

    def test_country_aliases(self):
        self.assertAliases(NorthMacedonia, MK, MKD)

    def test_2019(self):
        # https://en.wikipedia.org/wiki/Public_holidays_in_North_Macedonia
        self.assertHolidays(
            ("2019-01-01", "New Year's Day"),
            ("2019-01-07", "Christmas Day (Orthodox)"),
            ("2019-04-29", "Easter Monday (Orthodox)"),
            ("2019-05-01", "Labour Day"),
            ("2019-05-24", "Saints Cyril and Methodius Day"),
            ("2019-06-04", "Eid al-Fitr (estimated)"),
            ("2019-08-02", "Republic Day"),
            ("2019-09-08", "Independence Day"),
            ("2019-10-11", "Day of Macedonian Uprising in 1941"),
            ("2019-10-23", "Day of the Macedonian Revolutionary Struggle"),
            ("2019-12-08", "Saint Clement of Ohrid Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from collections.abc import Iterable
from datetime import date
from typing import Optional

from holidays.calendars import _HinduLunisolar
from holidays.groups.eastern import EasternCalendarHolidays


class HinduCalendarHolidays(EasternCalendarHolidays):
    """
    Hindu lunisolar calendar holidays.
    """

    def __init__(self, cls=None, show_estimated=False) -> None:
        self._hindu_calendar = cls() if cls else _HinduLunisolar()
        self._hindu_calendar_show_estimated = show_estimated

    def _add_hindu_calendar_holiday(
        self, name: str, dt_estimated: tuple[Optional[date], bool]
    ) -> Optional[date]:
        """
        Add Hindu calendar holiday.

        Adds customizable estimation label to holiday name if holiday date
        is an estimation.
        """

        return self._add_eastern_calendar_holiday(
            name, dt_estimated, self._hindu_calendar_show_estimated
        )

    def _add_hindu_calendar_holiday_set(
        self, name: str, dts_estimated: Iterable[tuple[date, bool]], days_delta: int = 0
    ) -> set[date]:
        """
        Add Hindu calendar holidays.

        Adds customizable estimation label to holiday name if holiday date
        is an estimation.
        """
        added_dates = set()
        for dt_estimated in dts_estimated:
            if dt := self._add_eastern_calendar_holiday(
                name, dt_estimated, self._hindu_calendar_show_estimated, days_delta=days_delta
            ):
                added_dates.add(dt)

        return added_dates

    def _add_buddha_purnima(self, name) -> Optional[date]:
        """
        Add Buddha Purnima.

        Buddha Purnima, also known as Vesak, commemorates the birth, enlightenment,
        and passing of Gautama Buddha. It falls on the full moon day of the
        Hindu month of Vaisakha (April-May).
        https://en.wikipedia.org/wiki/Vesak
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.buddha_purnima_date(self._year)
        )

    def _add_chhath_puja(self, name) -> Optional[date]:
        """
        Add Chhath Puja.

        Chhath Puja is a Hindu festival dedicated to the Sun God (Surya).
        It is observed six days after Diwali in the month of Kartika (October-November).
        https://en.wikipedia.org/wiki/Chhath
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.chhath_puja_date(self._year)
        )

    def _add_diwali(self, name) -> Optional[date]:
        """
        Add Diwali Festival.

        Diwali (Deepavali, Festival of Lights) is one of the most important
        festivals in Indian religions. It is celebrated during the Hindu
        lunisolar months of Ashvin and Kartika (between mid-October and
        mid-November).
        https://en.wikipedia.org/wiki/Diwali
        """
        return self._add_hindu_calendar_holiday(name, self._hindu_calendar.diwali_date(self._year))

    def _add_diwali_india(self, name) -> Optional[date]:
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.diwali_india_date(self._year)
        )

    def _add_dussehra(self, name) -> Optional[date]:
        """
        Add Dussehra Festival.

        Dussehra (Vijayadashami) is a major Hindu festival that marks the end
        of Navratri. It is celebrated on the 10th day of the Hindu lunisolar
        month of Ashvin (September-October).
        https://en.wikipedia.org/wiki/Vijayadashami
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.dussehra_date(self._year)
        )

    def _add_ganesh_chaturthi(self, name) -> Optional[date]:
        """
        Add Ganesh Chaturthi.

        Ganesh Chaturthi is a Hindu festival celebrating the birth of Lord Ganesha.
        It falls on the fourth day of the Hindu month of Bhadrapada (August/September).
        https://en.wikipedia.org/wiki/Ganesh_Chaturthi
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.ganesh_chaturthi_date(self._year)
        )

    def _add_govardhan_puja(self, name) -> Optional[date]:
        """
        Add Govardhan Puja.

        Govardhan Puja, also known as Annakut, is celebrated the day after Diwali
        to honor Lord Krishna. It falls on the first lunar day of the Hindu month of Kartika.
        https://en.wikipedia.org/wiki/Govardhan_Puja
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.govardhan_puja_date(self._year)
        )

    def _add_gudi_padwa(self, name) -> Optional[date]:
        """
        Add Gudi Padwa.

        Gudi Padwa is the traditional New Year festival for Maharashtrians.
        It falls on the first day of Chaitra (March-April).
        https://en.wikipedia.org/wiki/Gudi_Padwa
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.gudi_padwa_date(self._year)
        )

    def _add_guru_gobind_singh_jayanti(self, name) -> set[date]:
        """
        Add Guru Gobind Singh Jayanti.

        Guru Gobind Singh Jayanti commemorates the birth anniversary of
        Guru Gobind Singh, the tenth Sikh Guru. It follows the Nanakshahi calendar.
        https://en.wikipedia.org/wiki/Guru_Gobind_Singh
        """
        return self._add_hindu_calendar_holiday_set(
            name, self._hindu_calendar.guru_gobind_singh_jayanti_date(self._year)
        )

    def _add_guru_nanak_jayanti(self, name) -> Optional[date]:
        """
        Add Guru Nanak Jayanti.

        Guru Nanak Jayanti celebrates the birth anniversary of Guru Nanak,
        the founder of Sikhism. It is observed on the full moon day of
        Kartik (October-November).
        https://en.wikipedia.org/wiki/Guru_Nanak_Gurpurab
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.guru_nanak_jayanti_date(self._year)
        )

    def _add_holi(self, name) -> Optional[date]:
        """
        Add Holi Festival.

        Holi, known as the Festival of Colors, is a Hindu festival that marks
        the arrival of spring. It is celebrated on the full moon day of the
        Hindu month of Phalguna (February/March).
        https://en.wikipedia.org/wiki/Holi
        """
        return self._add_hindu_calendar_holiday(name, self._hindu_calendar.holi_date(self._year))

    def _add_janmashtami(self, name) -> Optional[date]:
        """
        Add Janmashtami.

        Janmashtami is a Hindu festival that celebrates the birth of Lord Krishna.
        It falls on the eighth day of the Hindu month of Bhadrapada (August/September).
        https://en.wikipedia.org/wiki/Krishna_Janmashtami
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.janmashtami_date(self._year)
        )

    def _add_maha_navami(self, name) -> Optional[date]:
        """
        Add Maha Navami.

        Maha Navami is the ninth day of Navratri, dedicated to Goddess Durga.
        It is observed in Ashvin (September-October).
        https://en.wikipedia.org/wiki/Navaratri
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.maha_navami_date(self._year)
        )

    def _add_maha_shivaratri(self, name) -> Optional[date]:
        """
        Add Maha Shivaratri.

        Maha Shivaratri is a Hindu festival dedicated to Lord Shiva. It is celebrated
        on the 14th night of the Hindu month of Phalguna (February/March).
        https://en.wikipedia.org/wiki/Maha_Shivaratri
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.maha_shivaratri_date(self._year)
        )

    def _add_mahavir_jayanti(self, name) -> Optional[date]:
        """
        Add Mahavir Jayanti.

        Mahavir Jayanti celebrates the birth of Lord Mahavira, the 24th
        Tirthankara of Jainism. It falls on the 13th day of Chaitra (March-April).
        https://en.wikipedia.org/wiki/Mahavir_Jayanti
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.mahavir_jayanti_date(self._year)
        )

    def _add_makar_sankranti(self, name) -> Optional[date]:
        """
        Add Makar Sankranti.

        Makar Sankranti is a Hindu festival that marks the transition of the Sun
        into Capricorn (Makar). It is celebrated on January 14th or 15th every year.
        https://en.wikipedia.org/wiki/Makar_Sankranti
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.makar_sankranti_date(self._year)
        )

    def _add_onam(self, name) -> Optional[date]:
        """
        Add Onam.

        Onam is a major festival in Kerala, celebrating the homecoming of
        King Mahabali. It falls in the month of Chingam (August-September).
        https://en.wikipedia.org/wiki/Onam
        """
        return self._add_hindu_calendar_holiday(name, self._hindu_calendar.onam_date(self._year))

    def _add_raksha_bandhan(self, name) -> Optional[date]:
        """
        Add Raksha Bandhan.

        Raksha Bandhan is a Hindu festival that celebrates the bond between
        brothers and sisters. It falls on the full moon day of the Hindu month
        of Shravana (July/August).
        https://en.wikipedia.org/wiki/Raksha_Bandhan
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.raksha_bandhan_date(self._year)
        )

    def _add_ram_navami(self, name) -> Optional[date]:
        """
        Add Ram Navami.

        Ram Navami is a Hindu festival celebrating the birth of Lord Rama.
        It is observed on the ninth day of the Hindu month of Chaitra (March/April).
        https://en.wikipedia.org/wiki/Rama_Navami
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.ram_navami_date(self._year)
        )

    def _add_sharad_navratri(self, name) -> Optional[date]:
        """
        Add Navratri / Sharad Navratri.

        Navratri is a Hindu festival dedicated to the worship of Goddess Durga.
        It is celebrated over nine nights and occurs in the lunar month of Ashvin
        (September/October).
        https://en.wikipedia.org/wiki/Navratri
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.sharad_navratri_date(self._year)
        )

    def _add_thaipusam(self, name) -> Optional[date]:
        """
        Add Thaipusam.

        Thaipusam is a Tamil Hindu festival celebrated on the full moon
        of the Tamil month of Thai (January/February).
        https://en.wikipedia.org/wiki/Thaipusam
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.thaipusam_date(self._year)
        )

    def _add_vaisakhi(self, name) -> Optional[date]:
        """
        Add Vaisakhi.

        Vaisakhi is a major Sikh festival marking the Sikh New Year and the
        founding of the Khalsa. It falls on April 13 or 14.
        https://en.wikipedia.org/wiki/Vaisakhi
        """
        return self._add_hindu_calendar_holiday(
            name, self._hindu_calendar.vaisakhi_date(self._year)
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import warnings
from unittest import TestCase

from holidays.countries.marshall_islands import HolidaysMH, MH, MHL
from tests.common import CommonCountryTests


class TestMH(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(HolidaysMH)

    def setUp(self):
        super().setUp()
        warnings.simplefilter("ignore")

    def test_country_aliases(self):
        self.assertAliases(HolidaysMH, MH, MHL)

    def test_2020(self):
        # http://web.archive.org/web/20201125072002/https://www.pscrmi.net/rmi-holiday-memos
        self.assertHolidays(
            HolidaysMH(years=2020),
            ("2020-01-01", "New Year's Day"),
            ("2020-03-01", "Nuclear Victims Remembrance Day"),
            ("2020-03-02", "Nuclear Victims Remembrance Day Holiday"),
            ("2020-04-10", "Good Friday"),
            ("2020-05-01", "Constitution Day"),
            ("2020-07-03", "Fisherman's Day"),
            ("2020-09-04", "Dri-jerbal Day"),
            ("2020-09-25", "Manit Day"),
            ("2020-11-17", "President's Day"),
            ("2020-12-04", "Gospel Day"),
            ("2020-12-25", "Christmas Day"),
        )

    def test_2021(self):
        # http://web.archive.org/web/20210617163816/https://www.pscrmi.net/rmi-holiday-memos
        self.assertHolidays(
            HolidaysMH(years=2021),
            ("2021-01-01", "New Year's Day"),
            ("2021-03-01", "Nuclear Victims Remembrance Day"),
            ("2021-04-02", "Good Friday"),
            ("2021-05-01", "Constitution Day"),
            ("2021-07-02", "Fisherman's Day"),
            ("2021-09-03", "Dri-jerbal Day"),
            ("2021-09-24", "Manit Day"),
            ("2021-11-17", "President's Day"),
            ("2021-12-03", "Gospel Day"),
            ("2021-12-24", "Christmas Day"),
        )

    def test_2022(self):
        # http://web.archive.org/web/20220704021442/https://www.pscrmi.net/rmi-holiday-memos
        self.assertHolidays(
            HolidaysMH(years=2022),
            ("2022-01-01", "New Year's Day"),
            ("2022-03-01", "Nuclear Victims Remembrance Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-05-01", "Constitution Day"),
            ("2022-05-02", "Constitution Day Holiday"),
            ("2022-07-01", "Fisherman's Day"),
            ("2022-09-02", "Dri-jerbal Day"),
            ("2022-09-30", "Manit Day"),
            ("2022-11-17", "President's Day"),
            ("2022-12-02", "Gospel Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day Holiday"),
        )

    def test_2023(self):
        # https://web.archive.org/web/20230628074915/https://www.pscrmi.net/rmi-holiday-memos
        self.assertHolidays(
            HolidaysMH(years=2023),
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day Holiday"),
            ("2023-03-01", "Nuclear Victims Remembrance Day"),
            ("2023-04-07", "Good Friday"),
            ("2023-05-01", "Constitution Day"),
            ("2023-07-07", "Fisherman's Day"),
            ("2023-09-01", "Dri-jerbal Day"),
            ("2023-09-29", "Manit Day"),
            ("2023-11-17", "President's Day"),
            ("2023-11-20", "General Election Day"),
            ("2023-12-01", "Gospel Day"),
            ("2023-12-25", "Christmas Day"),
        )

    def test_special_holidays(self):
        self.assertHoliday(
            # General Election are held on the 3rd Monday of November
            # see https://en.wikipedia.org/wiki/Elections_in_the_Marshall_Islands
            "1995-11-20",
            "1999-11-22",
            "2003-11-17",
            "2007-11-19",
            "2011-11-21",
            "2015-11-16",
            "2019-11-18",
            "2023-11-20",
        )

    def test_not_observed(self):
        self.assertNoNonObservedHoliday(
            "2020-03-02",
            "2022-05-02",
            "2022-12-26",
            "2023-01-02",
        )
